import os
import sys
import numpy as np
import urllib

scriptDirectory = os.path.dirname(os.path.abspath(__file__))

from restore_model import LoadInputSpectra, BestTypesListSingleRedshift

try:
    from PyQt4 import QtGui
    from main import MainApp
except ImportError:
    print "Warning: You will need to install 'PyQt4' if you want to use the graphical interface. " \
          "Using the automatic library will continue to work."



class Classify(object):
    def __init__(self, filenames=[], redshifts=[], smooth=15):
        """ Takes a list of filenames and corresponding redshifts for supernovae.
        Files should contain a single spectrum, and redshifts should be a list of corresponding redshift floats
        """
        self.filenames = filenames
        self.redshifts = redshifts
        self.smooth = smooth
        self.numSpectra = len(filenames)
        self.mainDirectory = os.path.dirname(os.path.abspath(__file__))

        modelFilename = os.path.join(scriptDirectory, 'model_trainedAtZeroZ.ckpt')
        if not os.path.isfile(modelFilename):
            print "Downloading Training Model..."
            modelFileDownload = urllib.URLopener()
            modelFileDownload.retrieve(
                "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/model_trainedAtZeroZ.ckpt",
                modelFilename)
            print modelFilename
        dataFilename = os.path.join(scriptDirectory, 'type_age_atRedshiftZero.npz')
        if not os.path.isfile(dataFilename):
            print "Downloading Data File..."
            dataFileDownload = urllib.URLopener()
            dataFileDownload.retrieve(
                "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/type_age_atRedshiftZero.npz",
                dataFilename)
            print dataFilename
        dataFilename = os.path.join(scriptDirectory, 'training_params.pickle')
        if not os.path.isfile(dataFilename):
            print "Downloading Data File..."
            dataFileDownload = urllib.URLopener()
            dataFileDownload.retrieve(
                "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/training_params.pickle",
                dataFilename)
            print dataFilename
        dataFilename = os.path.join(scriptDirectory, 'templates.npz')
        if not os.path.isfile(dataFilename):
            print "Downloading Data File..."
            dataFileDownload = urllib.URLopener()
            dataFileDownload.retrieve(
                "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/templates.npz", dataFilename)
            print dataFilename

        self.modelFilename = os.path.join(self.mainDirectory, "model_trainedAtZeroZ.ckpt")


    def _input_spectrum_info(self, filename, redshift, n):
        loadInputSpectra = LoadInputSpectra(filename, redshift, redshift, self.smooth)
        inputImage, inputRedshift, typeNamesList, nw, nBins = loadInputSpectra.input_spectra()
        bestTypesList = BestTypesListSingleRedshift(self.modelFilename, inputImage, typeNamesList, nw, nBins)
        bestTypes = bestTypesList.bestTypes
        softmax = bestTypesList.softmaxOrdered

        bestMatchList = []
        for i in range(n):
            name, age = bestTypes[i].split(': ')
            prob = softmax[i]
            bestMatchList.append((name, age, prob))

        return np.array(bestMatchList)

    def list_best_matches(self, n=1):
        """Returns a list of lists of the the top n best matches for each spectrum"""
        bestMatchLists = []
        for i in range(self.numSpectra):
            f = self.filenames[i]
            z = self.redshifts[i]
            bestMatchList = self._input_spectrum_info(f, z, n)
            bestMatchLists.append(bestMatchList)

        return np.array(bestMatchLists)

    def plot_with_gui(self, indexToPlot=0):
        app = QtGui.QApplication(sys.argv)
        form = MainApp(inputFilename=self.filenames[indexToPlot])
        form.lblInputFilename.setText(self.filenames[indexToPlot])
        form.lineEditKnownZ.setText(str(self.redshifts[indexToPlot]))
        form.lineEditSmooth.setText(str(self.smooth))
        form.fit_spectra()
        form.show()
        app.exec_()






# # EXAMPLE USAGE:
# classification = Classify(filenames=['/Users/dmuthukrishna/Users/dmuthukrishna/DES16E1dic_E1_combined_161125_v10_b00.dat',
#                                      '/Users/dmuthukrishna/Users/dmuthukrishna/DES16E1dic_E1_combined_161125_v10_b00.dat'],
#                           redshifts=[0.34, 0.13])
# print classification.list_best_matches()







import numpy as np
from random import shuffle
from sn_processing import PreProcessing


class AgeBinning(object):
    def __init__(self, minAge, maxAge, ageBinSize):
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize

    def age_bin(self, age):
        ageBin = int(round(age / self.ageBinSize)) - int(round(self.minAge / self.ageBinSize))

        return ageBin

    def age_labels(self):
        ageLabels = []

        ageBinPrev = 0
        ageLabelMin = self.minAge
        for age in np.arange(self.minAge, self.maxAge, 0.5):
            ageBin = self.age_bin(age)

            if (ageBin != ageBinPrev):
                ageLabelMax = int(round(age))
                ageLabels.append(str(ageLabelMin) + " to " + str(ageLabelMax))
                ageLabelMin = ageLabelMax

            ageBinPrev = ageBin

        ageLabels.append(str(ageLabelMin) + " to " + str(self.maxAge))

        return ageLabels


class CreateLabels(object):

    def __init__(self, nTypes, minAge, maxAge, ageBinSize, typeList):
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge-0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.ageLabels = self.ageBinning.age_labels()       
        

    def label_array(self, ttype, age):
        ageBin = self.ageBinning.age_bin(age)
        labelarray = np.zeros((self.nTypes, self.numOfAgeBins))
        typeNames = []

        try:
            typeIndex = self.typeList.index(ttype)
        except ValueError as err:
            print("INVALID TYPE: {0}".format(err))

            
        labelarray[typeIndex][ageBin] = 1
        labelarray = labelarray.flatten()

        typeNames.append(ttype + ": " + self.ageLabels[ageBin])
        typeNames = np.array(typeNames)

        return labelarray, typeNames


    def type_names_list(self):
        typeNamesList = []
        for tType in self.typeList:
            for ageLabel in self.ageBinning.age_labels():
                typeNamesList.append(tType + ": " + ageLabel)

        return typeNamesList
        

class TempList():
    def temp_list(self, tempFileList):
        f = open(tempFileList, 'rU')

        fileList = f.readlines()
        for i in range(0,len(fileList)):
            fileList[i] = fileList[i].strip('\n')

        f.close()

        return fileList
    

class ReadSpectra(object):

    def __init__(self, w0, w1, nw, filename):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.filename = filename
        self.data = PreProcessing(filename, self.w0, self.w1, self.nw)


    def snid_template_data(self, ageIdx, z):
        """ lnw template files """
        wave, flux, nCols, ages, tType, minIndex, maxIndex = self.data.snid_template_data(ageIdx, z)

        return wave, flux, nCols, ages, tType, minIndex, maxIndex


    def sf_age(self):
        snName, extension = self.filename.strip('.dat').split('.')
        ttype, snName = snName.split('/')

        if (extension == 'max'):
            age = 0
        elif (extension[0] == 'p'):
            age = float(extension[1:])
        elif (extension[0] == 'm'):
            age = -float(extension[1:])
        else:
            print "Invalid Superfit Filename: " + self.filename

        return snName, ttype, age


    def superfit_template_data(self, z):
        """ Returns wavelength and flux after all preprocessing """
        wave, flux, minIndex, maxIndex = self.data.two_column_data(z)
        snName, ttype, age = self.sf_age()

        print snName, ttype, age

        return wave, flux, minIndex, maxIndex, age, snName, ttype


    def input_spectrum(self, z, smooth):
        wave, flux, minIndex, maxIndex = self.data.two_column_data(z, smooth)

        return wave, flux, minIndex, maxIndex

class ArrayTools(object):

    def __init__(self, nLabels, nw):
        self.nLabels = nLabels
        self.nw = nw

    def shuffle_arrays(self, images, labels, filenames, typeNames):
        arraySize = len(labels)
        imagesShuf = np.empty((arraySize, int(self.nw)), np.float16)
        labelsShuf = np.empty((arraySize, self.nLabels), np.uint8)
        filenamesShuf = np.empty(arraySize, dtype=object)
        typeNamesShuf = np.empty(arraySize, dtype=object)
        idx = 0
        print("Shuffle2")
        # Randomise order
        indexShuf = range(arraySize)
        shuffle(indexShuf)
        for i in indexShuf:
            imagesShuf[idx] = images[i]
            labelsShuf[idx] = labels[i]
            filenamesShuf[idx] = filenames[i]
            typeNamesShuf[idx] = typeNames[i]
            idx += 1
        print ("LenLabels")
        print(len(labels), idx)
            
        print(imagesShuf)
        print(typeNamesShuf)
        print("Shuffle3")    
        return imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf


    def count_labels(self, labels):
        counts = np.zeros(self.nLabels)

        for i in range(len(labels)):
            counts = labels[i] + counts

        return counts


    def div0(self, a, b):
        """ ignore / 0, div0( [-1, 0, 1], 0 ) -> [0, 0, 0] """
        with np.errstate(divide='ignore', invalid='ignore'):
            c = np.true_divide(a, b)
            c[~ np.isfinite(c)] = 0  # -inf inf NaN
        return c


    def over_sample_arrays(self, images, labels, filenames, typeNames):
        counts = self.count_labels(labels)
        idx = 0
        print "Before OverSample"  #
        print counts  #


        overSampleAmount = self.div0(1 * max(counts), counts)  # ignore zeros in counts
        overSampleArraySize = int(sum(np.array(overSampleAmount, int) * counts))
        print(np.array(overSampleAmount, int) * counts)
        print(np.array(overSampleAmount, int))
        print(overSampleArraySize, len(labels))
        imagesOverSampled = np.zeros((overSampleArraySize, int(self.nw)), np.float16)
        labelsOverSampled = np.zeros((overSampleArraySize, self.nLabels), np.uint8)
        filenamesOverSampled = np.empty(overSampleArraySize, dtype=object)
        typeNamesOverSampled = np.empty(overSampleArraySize, dtype=object)
        

        counts1 = np.zeros(self.nLabels)

        imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf = self.shuffle_arrays(images, labels, filenames, typeNames)
        print(len(labelsShuf))
        for i in range(len(labelsShuf)):
            label = labelsShuf[i]
            image = imagesShuf[i]
            filename = filenamesShuf[i]
            typeName = typeNamesShuf[i]

            labelIndex = np.argmax(label)
            
            print(idx, i, int(overSampleAmount[labelIndex]))
            for r in range(int(overSampleAmount[labelIndex])):
                imagesOverSampled[idx] = image #np.concatenate(imagesOverSampled, np.array([image]), axis=0)
                labelsOverSampled[idx] = label #np.concatenate(labelsOverSampled, np.array([label]), axis=0)
                filenamesOverSampled[idx] = filename #filenamesOverSampled = np.append(filenamesOverSampled, filename)
                typeNamesOverSampled[idx] = typeName #typeNamesOverSampled = np.append(typeNamesOverSampled, typeName)
                counts1 = label + counts1
                idx += 1
            

                
        print "After OverSample"  #
        print counts1  #

        print("Before Shuffling")
        imagesOverSampledShuf, labelsOverSampledShuf, filenamesOverSampledShuf, typeNamesOverSampledShuf = self.shuffle_arrays(imagesOverSampled, labelsOverSampled, filenamesOverSampled, typeNamesOverSampled)
        print("After Shuffling")
        return imagesOverSampledShuf, labelsOverSampledShuf, filenamesOverSampledShuf, typeNamesOverSampledShuf


class CreateArrays(object):
    def __init__(self, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.minZ = minZ
        self.maxZ = maxZ
        self.redshiftPrecision = 50
        self.numOfRedshifts = (self.maxZ - self.minZ) * self.redshiftPrecision
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge-0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.createLabels = CreateLabels(self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList)

    def snid_templates_to_arrays(self, snidTemplateLocation, tempfilelist):
        ''' This function is for the SNID processed files, which
            have been preprocessed to negatives, and so cannot be
            imaged yet '''

        templist = TempList().temp_list(tempfilelist) #Arbrirary redshift to read filelist
        typeList = []
        images = np.empty((0, int(self.nw)), np.float32)  # Number of pixels
        labels = np.empty((0, self.nLabels), np.uint8)  # Number of labels (SN types)
        filenames = []#np.empty(0)
        typeNames = []#np.empty(0)
        agesList = []

        for i in range(0, len(templist)):
            ncols = 15
            readSpectra = ReadSpectra(self.w0, self.w1, self.nw, snidTemplateLocation + templist[i])
            
            for ageidx in range(0, 100):
                if (ageidx < ncols):
                    for z in np.linspace(self.minZ, self.maxZ, self.numOfRedshifts + 1):
                        tempwave, tempflux, ncols, ages, ttype, tminindex, tmaxindex = readSpectra.snid_template_data(ageidx, z)
                        agesList.append(ages[ageidx])
                        
                    
                        if ((float(ages[ageidx]) > self.minAge and float(ages[ageidx]) < self.maxAge)):
                            label, typeName = self.createLabels.label_array(ttype, ages[ageidx])
                            nonzeroflux = tempflux[tminindex:tmaxindex + 1]
                            newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
                            newflux2 = np.concatenate((tempflux[0:tminindex], newflux, tempflux[tmaxindex + 1:]))
                            images = np.append(images, np.array([newflux2]), axis=0)  # images.append(newflux2)
                            labels = np.append(labels, np.array([label]), axis=0)  # labels.append(ttype)
                            filenames.append(templist[i] + '_' + ttype + '_' + str(ages[ageidx]) + '_z' + str(z))
                            typeNames.append(typeName)

            print templist[i]
            # Create List of all SN types
            if ttype not in typeList:
                typeList.append(ttype)

        return typeList, images, labels, np.array(filenames), np.array(typeNames)

    def superfit_templates_to_arrays(self, sfTemplateLocation, sftempfilelist):
        templist = TempList().temp_list(sftempfilelist)
        images = np.empty((0, self.nw), np.float32)  # Number of pixels
        labels = np.empty((0, self.nLabels), np.float32)  # Number of labels (SN types)
        filenames = []
        typeNames = []

        for i in range(0, len(templist)):
            readSpectra = ReadSpectra(self.w0, self.w1, self.nw, sfTemplateLocation + templist[i])
            tempwave, tempflux, tminindex, tmaxindex, age, snName, ttype = self.readSpectra.superfit_template_data(z)

            if ((float(ages[ageidx]) > minAge and float(ages[ageidx]) > maxAge)):
                label, typeName = label_array(ttype, ages[ageidx])
                nonzeroflux = tempflux[tminindex:tmaxindex + 1]
                newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
                newflux2 = np.concatenate((tempflux[0:tminindex], newflux, tempflux[tmaxindex + 1:]))
                images = np.append(images, np.array([newflux2]), axis=0)  # images.append(newflux2)
                labels = np.append(labels, np.array([label]), axis=0)  # labels.append(ttype)
                filenames.append(templist[i])
                typeNames.append(typeName)

        return images, labels, np.array(filenames), typeNames





"""Functions for downloading and reading MNIST data."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import gzip
import os
import tempfile

import numpy
from six.moves import urllib
from six.moves import xrange  # pylint: disable=redefined-builtin
import tensorflow as tf
from tensorflow.contrib.learn.python.learn.datasets.mnist import read_data_sets

#10 classes
#784 (28x28) pixels in each image
#None (55000) images

####################################################
#mnist = read_data_sets("MNIST_data/", one_hot=True)

import matplotlib.pyplot as plt
import numpy as np
import itertools
import tensorflow as tf


loaded = np.load('type_age_atRedshiftZero.npz')
trainImages = loaded['trainImages']
trainLabels = loaded['trainLabels']
#trainFilenames = loaded['trainFilenames']
#trainTypeNames = loaded['trainTypeNames']
testImages = loaded['testImages']
testLabels = loaded['testLabels']
#testFilenames = loaded['testFilenames']
testTypeNames = loaded['testTypeNames']
typeNamesList = loaded['typeNamesList']
#validateImages = sortData[2][0]
#validateLabels = sortData[2][1]


print("Completed creatingArrays")

N = 1024
ntypes = len(testLabels[0])
imWidth = 32 #Image size and width
imWidthReduc = 8

a = []

sess = tf.InteractiveSession()

#WEIGHT INITIALISATION
def weight_variable(shape):
    initial = tf.truncated_normal(shape, stddev=0.1)
    return tf.Variable(initial)

def bias_variable(shape):
    initial = tf.constant(0.1, shape=shape)
    return tf.Variable(initial)

#CONVOLUTION AND POOLING
def conv2d(x, W):
    return tf.nn.conv2d(x, W, strides=[1, 1, 1, 1], padding='SAME')

def max_pool_2x2(x):
    return tf.nn.max_pool(x, ksize=[1, 2, 2, 1], strides=[1, 2, 2, 1], padding='SAME')

x = tf.placeholder(tf.float32, shape=[None, N])
y_ = tf.placeholder(tf.float32, shape=[None, ntypes])

#FIRST CONVOLUTIONAL LAYER
W_conv1 = weight_variable([5, 5, 1, 32])
b_conv1 = bias_variable([32])
x_image = tf.reshape(x, [-1,imWidth,imWidth,1])
h_conv1 = tf.nn.relu(conv2d(x_image, W_conv1) + b_conv1)
h_pool1 = max_pool_2x2(h_conv1)

#SECOND CONVOLUTIONAL LAYER
W_conv2 = weight_variable([5, 5, 32, 64])
b_conv2 = bias_variable([64])
h_conv2 = tf.nn.relu(conv2d(h_pool1, W_conv2) + b_conv2)
h_pool2 = max_pool_2x2(h_conv2)

#DENSELY CONNECTED LAYER
W_fc1 = weight_variable([imWidthReduc * imWidthReduc * 64, N])
b_fc1 = bias_variable([N])
h_pool2_flat = tf.reshape(h_pool2, [-1, imWidthReduc*imWidthReduc*64])
h_fc1 = tf.nn.relu(tf.matmul(h_pool2_flat, W_fc1) + b_fc1)

#DROPOUT
keep_prob = tf.placeholder(tf.float32)
h_fc1_drop = tf.nn.dropout(h_fc1, keep_prob)

#READOUT LAYER
W_fc2 = weight_variable([N, ntypes])
b_fc2 = bias_variable([ntypes])

y_conv = tf.nn.softmax(tf.matmul(h_fc1_drop, W_fc2) + b_fc2)

########
##batch_xs1 = trainImages
##batch_ys1 = trainLabels
##print(sess.run(y_conv, feed_dict={x: batch_xs1, y_: batch_ys1}))
########

#TRAIN AND EVALUATE MODEL
cross_entropy = tf.reduce_mean(-tf.reduce_sum(y_ * tf.log(y_conv), reduction_indices=[1]))
train_step = tf.train.AdamOptimizer(1e-4).minimize(cross_entropy)
correct_prediction = tf.equal(tf.argmax(y_conv,1), tf.argmax(y_,1))
accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))

sess.run(tf.initialize_all_variables())

trainImagesCycle = itertools.cycle(trainImages)
trainLabelsCycle = itertools.cycle(trainLabels)
for i in range(100000):
    batch_xs = np.array(list(itertools.islice(trainImagesCycle, 50*i, 50*i+50)))
    batch_ys = np.array(list(itertools.islice(trainLabelsCycle, 50*i, 50*i+50)))
    train_step.run(feed_dict={x: batch_xs, y_: batch_ys, keep_prob: 0.5})
    if (i % 100 == 0):
        train_accuracy = accuracy.eval(feed_dict={x:batch_xs, y_: batch_ys, keep_prob: 1.0})
        print("step %d, training accuracy %g"%(i, train_accuracy))
        testacc = accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
        print("test accuracy %g"%(testacc))
        a.append(testacc)
print("test accuracy %g"%accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0}))

yy = y_conv.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
cp = correct_prediction.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
print(cp)
for i in range(len(cp)):
    if (cp[i] == False):
        predictedIndex = np.argmax(yy[i])
        print(i, testTypeNames[i], typeNamesList[predictedIndex])


#ACTUAL ACCURACY, SUBTYPE ACCURACY, AGE ACCURACY
typeAndAgeCorrect = 0
typeCorrect = 0
subTypeCorrect = 0
subTypeAndAgeCorrect = 0
typeAndNearAgeCorrect = 0
subTypeAndNearAgeCorrect = 0
for i in range(len(testTypeNames)):
    predictedIndex = np.argmax(yy[i])
    testSubType = testTypeNames[i][0:2]
    actualSubType = typeNamesList[predictedIndex][0:2]
    if testTypeNames[i][0:3] == 'IIb':
        testSubType = 'Ib'
    if typeNamesList[predictedIndex][0:3] == 'IIb':
        actualSubType = 'Ib'
    testType = testTypeNames[i].split(': ')[0]
    actualType = typeNamesList[predictedIndex].split(': ')[0]
    testAge = testTypeNames[i].split(': ')[1]
    actualAge = typeNamesList[predictedIndex].split(': ')[1]
    nearTestAge = testAge.split(' to ')
    
    if (testTypeNames[i] == typeNamesList[predictedIndex]):
        typeAndAgeCorrect += 1
    if (testType == actualType): #correct type
        typeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge)): #check if the age is in the neigbouring bin
            typeAndNearAgeCorrect += 1 #all correct except nearby bin
    if (testSubType == actualSubType): #correct subtype
        subTypeCorrect += 1
        if testAge == actualAge:
            subTypeAndAgeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge)): #check if the age is in the neigbouring bin
            subTypeAndNearAgeCorrect += 1 #subtype and nearby bin

typeAndAgeAccuracy = float(typeAndAgeCorrect)/len(testTypeNames)
typeAccuracy = float(typeCorrect)/len(testTypeNames)
subTypeAccuracy = float(subTypeCorrect)/len(testTypeNames)
subTypeAndAgeAccuracy = float(subTypeAndAgeCorrect)/len(testTypeNames)
typeAndNearAgeAccuracy = float(typeAndNearAgeCorrect)/len(testTypeNames)
subTypeAndNearAgeAccuracy = float(subTypeAndNearAgeCorrect)/len(testTypeNames)

print("typeAndAgeAccuracy : " + str(typeAndAgeAccuracy))
print("typeAccuracy : " + str(typeAccuracy))
print("subTypeAccuracy : " + str(subTypeAccuracy))
print("subTypeAndAgeAccuracy: " + str(subTypeAndAgeAccuracy))
print("typeAndNearAgeAccuracy : " + str(typeAndNearAgeAccuracy))
print("subTypeAndNearAgeAccuracy : " + str(subTypeAndNearAgeAccuracy))
    

#SAVE THE MODEL
saver = tf.train.Saver()
save_path = saver.save(sess, "model_trainedAtZeroZ.ckpt")
print("Model saved in file: %s" % save_path)



#Pre-processing class

from preprocessing import *
#import matplotlib.pyplot as plt
from scipy.signal import medfilt



class PreProcessing(object):
    """ Pre-processes spectra for cross correlation """
    
    def __init__(self, filename, w0, w1, nw):
        self.filename = filename
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.numSplinePoints = 13
        self.readSpectrumFile = ReadSpectrumFile(filename, w0, w1, nw)
        self.preProcess = PreProcessSpectrum(w0, w1, nw)

        self.spectrum = self.readSpectrumFile.file_extension()

    def galaxy_template(self, z):
        self.wave, self.flux = self.spectrum
        wave, flux = self.readSpectrumFile.two_col_input_spectrum(self.wave, self.flux, z)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        newflux, continuum = self.preProcess.continuum_removal(binnedwave, binnedflux, self.numSplinePoints, minindex, maxindex)
        meanzero = self.preProcess.mean_zero(binnedwave, newflux, minindex, maxindex)
        apodized = self.preProcess.apodize(binnedwave, meanzero, minindex, maxindex)

        plt.figure('Galaxy_SB1')
        plt.plot(wave,flux, label='original')
        plt.plot(binnedwave,binnedflux, label='binned')
        plt.plot(binnedwave, continuum, label='continuum')
        plt.plot(binnedwave,newflux, label='continuumSubtracted')
        plt.plot(binnedwave, meanzero, label='meanzero')
        plt.plot(binnedwave,apodized, label='apodized')
        plt.legend()
        #plt.show()

        return binnedwave, apodized, minindex, maxindex


    def two_column_data(self, z, smooth):
        self.wave, self.flux = self.spectrum

        filterSize = int(len(self.wave)/self.nw) * smooth * 2 + 1
        preFiltered = medfilt(self.flux, kernel_size=filterSize)
        wave, flux = self.readSpectrumFile.two_col_input_spectrum(self.wave, preFiltered, z)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        newflux, continuum = self.preProcess.continuum_removal(binnedwave, binnedflux, self.numSplinePoints, minindex, maxindex)
        meanzero = self.preProcess.mean_zero(binnedwave, newflux, minindex, maxindex)
        apodized = self.preProcess.apodize(binnedwave, meanzero, minindex, maxindex)


        #filterSize = smooth * 2 + 1
        medianFiltered = medfilt(apodized, kernel_size=1)#filterSize)


        # from scipy.interpolate import interp1d
        #
        # plt.plot(self.flux)
        #
        # spline = interp1d(binnedwave[minindex:maxindex], binnedflux[minindex:maxindex], kind='cubic')
        # waveSpline = np.linspace(binnedwave[minindex],binnedwave[maxindex-1],num=self.numSplinePoints)
        # print spline
        # print '###'
        # print spline(binnedwave[minindex:maxindex])
        # plt.figure('1')
        # plt.plot(waveSpline, spline(waveSpline), '--', label='spline')
        #
        # print wave
        # print binnedwave
        # print binnedflux
        # print len(binnedwave)
        # plt.plot(wave,flux)
        # plt.figure('2')
        # plt.plot(binnedwave, binnedflux, label='binned')
        # plt.plot(binnedwave, newflux, label='continuumSubtract1')
        # plt.plot(binnedwave, continuum, label='polyfit1')
        # print len(binnedwave)
        # print (min(binnedwave), max(binnedwave))
        # print len(newflux)
        #
        # #newflux2, poly2 = self.preProcess.continuum_removal(binnedwave, binnedflux, 6, minindex, maxindex)
        # #plt.plot(binnedwave, newflux2, label='continuumSubtract2')
        # #plt.plot(binnedwave, poly2, label='polyfit2')
        # plt.plot(binnedwave, apodized, label='taper')
        # plt.legend()
        # plt.figure('filtered')
        # plt.plot(medianFiltered)
        # plt.figure('3')
        # plt.plot(medfilt(apodized,kernel_size=3))
        # plt.show()

        return binnedwave, medianFiltered, minindex, maxindex

    def snid_template_data(self, ageidx, z):
        """lnw templates """
        wave, fluxes, ncols, ages, ttype, splineInfo = self.spectrum
        wave, flux = self.readSpectrumFile.snid_template_spectra(wave, fluxes[ageidx], z, splineInfo)
        binnedwave, binnedflux, minindex, maxindex = self.preProcess.log_wavelength(wave, flux)
        medianFiltered = medfilt(binnedflux, kernel_size=5)

        return binnedwave, medianFiltered, ncols, ages, ttype, minindex, maxindex

if __name__ == '__main__':
    # fData = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/superfit_templates/sne/Ia/sn2002bo.m01.dat'
    # preData = PreProcessing(fGal, 2500, 10000, 1024)
    # waveData,fluxData,minIData,maxIData = preData.two_column_data(0, 0)

    fGal = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/galaxy_templates/GalaxyTemplates/Sa'
    preGal = PreProcessing(fGal, 2500, 10000, 1024)
    waveGal,fluxGal,minIGal,maxIGal = preGal.galaxy_template(0)

    fSN = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/snid_templates_Modjaz_BSNIP/sn2001br.lnw'
    preSN = PreProcessing(fSN, 2500, 10000, 1024)
    waveSN, fluxSN, ncols, ages, ttype, minISN, maxISN = preSN.snid_template_data(0, 0)

    plt.figure('SN and Galaxy Template')
    minI = max(minIGal, minISN)
    maxI = min(maxIGal, maxISN)
    plt.axvline(x=waveSN[minI], color='k', linestyle='--')
    plt.axvline(x=waveSN[maxI], color='k', linestyle='--')



    plt.plot(waveGal,fluxGal, label='galaxy')
    plt.plot(waveSN,fluxSN, label='SN')

    waveSN = waveSN[minI:maxI]
    fluxSN = fluxSN[minI:maxI]
    fluxGal = fluxGal[minI:maxI]

    plt.plot(waveSN,fluxGal+fluxSN, label='Added-50%')
    plt.legend()
    plt.show()



import dash

filenames = []
filenames.append('DES16C3elb_C3_combined_161227_v10_b00.dat')
filenames.append('DES16X3dvb_X3_combined_161225_v10_b00.dat')
filenames.append('DES16C2ege_C2_combined_161225_v10_b00.dat')
filenames.append('DES16X3eww_X3_combined_161225_v10_b00.dat')
filenames.append('DES16X3enk_X3_combined_161225_v10_b00.dat')
filenames.append('DES16S1ffb_S1_combined_161226_v10_b00.dat')
filenames.append('DES16C1fgm_C1_combined_161226_v10_b00.dat')
filenames.append('DES16X2dzz_X2_combined_161226_v10_b00.dat')
filenames.append('DES16X1few_X1_combined_161227_v10_b00.dat')
filenames.append('DES16X1chc_X1_combined_161227_v10_b00.dat')
filenames.append('DES16S2ffk_S2_combined_161227_v10_b00.dat')


knownRedshifts = []
knownRedshifts.append(0.429)
knownRedshifts.append(0.329)
knownRedshifts.append(0.348)
knownRedshifts.append(0.445)
knownRedshifts.append(0.331)
knownRedshifts.append(0.164)
knownRedshifts.append(0.361)
knownRedshifts.append(0.325)
knownRedshifts.append(0.311)
knownRedshifts.append(0.043)
knownRedshifts.append(0.373)

classification = dash.Classify(filenames, knownRedshifts)
print classification.list_best_matches(n=3)
classification.plot_with_gui(indexToPlot=1)


import tensorflow as tf

class ConvNetLayer(object):
    def __init__(self, N, ntypes, imWidth, imWidthReduc):
        self.N = N
        self.ntypes = ntypes
        self.imWidth = imWidth
        self.imWidthReduc = imWidthReduc

    def build_layer(self, prevHPool, prevNumFeatures, numFeatures):
        W_conv = self._weight_variable([5, 5, prevNumFeatures, numFeatures])
        b_conv = self._bias_variable([numFeatures])
        h_conv = tf.nn.relu(self._conv2d(prevHPool, W_conv) + b_conv)
        h_pool = self._max_pool_2x2(h_conv)
        # print h_pool

        return h_pool

    def connect_layers(self, h_pool, numFeatures, layerNum):
        W_fc = self._weight_variable([self.imWidthReduc/layerNum * self.imWidthReduc/layerNum * numFeatures, 1024])
        b_fc = self._bias_variable([1024])
        h_pool_flat = tf.reshape(h_pool, [-1, self.imWidthReduc/layerNum * self.imWidthReduc/layerNum * numFeatures])
        h_fc = tf.nn.relu(tf.matmul(h_pool_flat, W_fc) + b_fc)

        return h_fc

    def dropout(self, h_fc):
        keep_prob = tf.placeholder(tf.float32)
        h_fc_drop = tf.nn.dropout(h_fc, keep_prob)

        return keep_prob, h_fc_drop

    def readout_layer(self):
        W_fc = self._weight_variable([1024, self.ntypes])
        b_fc = self._bias_variable([self.ntypes])

        return W_fc, b_fc

    def _weight_variable(self, shape):
        initial = tf.truncated_normal(shape, stddev=0.1)
        return tf.Variable(initial)

    def _bias_variable(self, shape):
        initial = tf.constant(0.1, shape=shape)
        return tf.Variable(initial)

    def _conv2d(self, x, W):
        return tf.nn.conv2d(x, W, strides=[1, 1, 1, 1], padding='SAME')

    def _max_pool_2x2(self, x):
        return tf.nn.max_pool(x, ksize=[1, 2, 2, 1], strides=[1, 2, 2, 1], padding='SAME')


def convnet_variables(imWidth, imWidthReduc, N, ntypes):
    x = tf.placeholder(tf.float32, shape=[None, N])
    y_ = tf.placeholder(tf.float32, shape=[None, ntypes])

    x_image = tf.reshape(x, [-1, imWidth, imWidth, 1])

    Layer1 = ConvNetLayer(N, ntypes, imWidth, imWidthReduc)
    h_pool1 = Layer1.build_layer(x_image, 1, 32)

    Layer2 = ConvNetLayer(N, ntypes, imWidth, imWidthReduc)
    h_pool2 = Layer2.build_layer(h_pool1, 32, 64)
    h_fc1 = Layer2.connect_layers(h_pool2, 64, 1)

    Layer3 = ConvNetLayer(N, ntypes, imWidth, imWidthReduc)
    h_pool3 = Layer3.build_layer(h_pool2, 64, 64)
    h_fc2 = Layer3.connect_layers(h_pool3, 64, 2)

    # READOUT LAYER
    keep_prob, h_fc2_drop = Layer3.dropout(h_fc1)
    W_fc3, b_fc3 = Layer3.readout_layer()
    y_conv = tf.nn.softmax(tf.matmul(h_fc2_drop, W_fc3) + b_fc3)

    return x, y_, keep_prob, y_conv



from create_arrays import *
import pickle

class CreateTrainingSet(object):

    def __init__(self, snidTemplateLocation, snidtempfilelist, sfTemplateLocation, sftempfilelist, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ):
        self.snidTemplateLocation = snidTemplateLocation
        self.snidtempfilelist = snidtempfilelist
        self.sfTemplateLocation = sfTemplateLocation
        self.sftempfilelist = sftempfilelist
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge-0.1) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.createArrays = CreateArrays(w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ)
        self.arrayTools = ArrayTools(self.nLabels, self.nw)
        
    def type_amounts(self, labels):
        counts = self.arrayTools.count_labels(labels)

        return counts


    def all_templates_to_arrays(self):
        images = np.empty((0, int(self.nw)), np.float32)  # Number of pixels
        labels = np.empty((0, self.nTypes), float)  # Number of labels (SN types)
        typeList = []

        typelistSnid, imagesSnid, labelsSnid, filenamesSnid, typeNamesSnid = self.createArrays.snid_templates_to_arrays(self.snidTemplateLocation, self.snidtempfilelist)
        # imagesSuperfit, labelsSuperfit, filenamesSuperfit, typeNamesSuperfit = superfit_templates_to_arrays(self.sfTemplateLocation, sftempfilelist)

        images = np.vstack((imagesSnid))  # , imagesSuperfit)) #Add in other templates from superfit etc.
        labels = np.vstack((labelsSnid))  # , labelsSuperfit))
        filenames = np.hstack((filenamesSnid))  # , filenamesSuperfit))
        typeNames = np.hstack((typeNamesSnid))

        typeList = typelistSnid

        imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf = self.arrayTools.shuffle_arrays(images, labels, filenames, typeNames)  # imagesShortlist, labelsShortlist = shortlist_arrays(images, labels)

        typeAmounts = self.type_amounts(labels)
        
        return typeList, imagesShuf, labelsShuf, filenamesShuf, typeNamesShuf, typeAmounts  # imagesShortlist, labelsShortlist


    def sort_data(self):
        trainPercentage = 0.8
        testPercentage = 0.2
        validatePercentage = 0.

        typeList, images, labels, filenames, typeNames, typeAmounts = self.all_templates_to_arrays()
        # imagesSuperfit, labelsSuperfit, filenamesSuperfit = superfit_templates_to_arrays(sftempfilelist)
        # imagesSuperfit, labelsSuperfit, filenamesSuperfit = shuffle_arrays(imagesSuperfit, labelsSuperfit, filenamesSuperfit)


        trainSize = int(trainPercentage * len(images))
        testSize = int(testPercentage * len(images))

        trainImages = images[:trainSize]
        testImages = images[trainSize: trainSize + testSize]
        validateImages = images[trainSize + testSize:]
        trainLabels = labels[:trainSize]
        testLabels = labels[trainSize: trainSize + testSize]
        validateLabels = labels[trainSize + testSize:]
        trainFilenames = filenames[:trainSize]
        testFilenames = filenames[trainSize: trainSize + testSize]
        validateFilenames = filenames[trainSize + testSize:]
        trainTypeNames = typeNames[:trainSize]
        testTypeNames = typeNames[trainSize: trainSize + testSize]
        validateTypeNames = typeNames[trainSize + testSize:]

        trainImagesOverSample, trainLabelsOverSample, trainFilenamesOverSample, trainTypeNamesOverSample = self.arrayTools.over_sample_arrays(trainImages, trainLabels, trainFilenames, trainTypeNames)
        testImagesShortlist, testLabelsShortlist, testFilenamesShortlist, testTypeNamesShortlist = testImages, testLabels, testFilenames, testTypeNames  # (testImages, testLabels, testFilenames)

        return ((trainImagesOverSample, trainLabelsOverSample, trainFilenamesOverSample, trainTypeNamesOverSample),
                (testImagesShortlist, testLabelsShortlist, testFilenamesShortlist, testTypeNamesShortlist),
                (validateImages, validateLabels, validateFilenames, validateTypeNames),
                typeAmounts)


class SaveTrainingSet(object):
    def __init__(self, snidTemplateLocation, snidtempfilelist, sfTemplateLocation, sftempfilelist, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ):
        self.snidTemplateLocation = snidTemplateLocation
        self.snidtempfilelist = snidtempfilelist
        self.sfTemplateLocation = sfTemplateLocation
        self.sftempfilelist = sftempfilelist
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.createLabels = CreateLabels(self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList)
        
        self.createTrainingSet = CreateTrainingSet(self.snidTemplateLocation, self.snidtempfilelist, self.sfTemplateLocation, self.sftempfilelist, self.w0, self.w1, self.nw, self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList, minZ, maxZ)
        self.sortData = self.createTrainingSet.sort_data()
        self.trainImages = self.sortData[0][0]
        self.trainLabels = self.sortData[0][1]
        self.trainFilenames = self.sortData[0][2]
        self.trainTypeNames = self.sortData[0][3]
        self.testImages = self.sortData[1][0]
        self.testLabels = self.sortData[1][1]
        self.testFilenames = self.sortData[1][2]
        self.testTypeNames = self.sortData[1][3]
        self.validateImages = self.sortData[2][0]
        self.validateLabels = self.sortData[2][1]
        self.validateFilenames = self.sortData[2][2]
        self.validateTypeNames = self.sortData[2][3]
        self.typeAmounts = self.sortData[3]

        self.typeNamesList = self.createLabels.type_names_list()


    def type_amounts(self):
        for i in range(len(self.typeNamesList)):
            print(str(self.typeAmounts[i]) + ": " + str(self.typeNamesList[i]))
        return self.typeNamesList, self.typeAmounts

    def save_arrays(self):
        saveFilename = 'type_age_atRedshiftZero.npz'
        np.savez_compressed(saveFilename, trainImages=self.trainImages, trainLabels=self.trainLabels,
                        trainFilenames=self.trainFilenames, trainTypeNames=self.trainTypeNames,
                        testImages=self.testImages, testLabels=self.testLabels,
                        testFilenames=self.testFilenames, testTypeNames=self.testTypeNames,
                        typeNamesList = self.typeNamesList)
        print("Saved Training Set to: " + saveFilename)
        

if __name__ == '__main__':
    with open('training_params.pickle') as f:
        nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList = pickle.load(f)
    minZ = 0
    maxZ = 0.0
    import os

    scriptDirectory = os.path.dirname(os.path.abspath(__file__))

    snidTemplateLocation = os.path.join(scriptDirectory, "../templates/snid_templates_Modjaz_BSNIP/")
    sfTemplateLocation = 'NOTUSED'
    snidtempfilelist1 = snidTemplateLocation + 'templist.txt'
    sftempfilelist1 = sfTemplateLocation + 'templist.txt'

    saveTrainingSet = SaveTrainingSet(snidTemplateLocation, snidtempfilelist1, sfTemplateLocation, sftempfilelist1, w0, w1, nw, nTypes, minAge, maxAge, ageBinSize, typeList, minZ, maxZ)
    typeNamesList, typeAmounts = saveTrainingSet.type_amounts()

    saveTrainingSet.save_arrays()



import numpy as np
import pickle
from create_arrays import *


class InputSpectra(object):
    def __init__(self, filename, minZ, maxZ, nTypes, minAge, maxAge, ageBinSize, w0, w1, nw, typeList, smooth):
        self.filename = filename
        self.minZ = minZ
        self.maxZ = maxZ
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.nTypes = nTypes
        self.minAge = minAge
        self.maxAge = maxAge
        self.ageBinSize = ageBinSize
        self.typeList = typeList
        self.ageBinning = AgeBinning(self.minAge, self.maxAge, self.ageBinSize)
        self.numOfAgeBins = self.ageBinning.age_bin(self.maxAge) + 1
        self.nLabels = self.nTypes * self.numOfAgeBins
        self.redshiftPrecision = 1000
        self.numOfRedshifts = (self.maxZ - self.minZ) * self.redshiftPrecision
        self.createLabels = CreateLabels(self.nTypes, self.minAge, self.maxAge, self.ageBinSize, self.typeList)
        self.fileType = 'fits or twocolumn etc.' #Will use later on
        self.typeNamesList = self.createLabels.type_names_list()
        self.smooth = smooth

    def redshifting(self):
        images = np.empty((0, int(self.nw)), np.float32)  # Number of pixels
        labels = np.empty((0, self.nLabels), float)  # Number of labels (SN types)
        filenames = []
        typeNames = []
        redshifts = []
        readSpectra = ReadSpectra(self.w0, self.w1, self.nw, self.filename)

        #Undo it's previous redshift)
        for z in np.linspace(self.minZ, self.maxZ, self.numOfRedshifts + 1):
            tempwave, tempflux, tminindex, tmaxindex = readSpectra.input_spectrum(z, self.smooth)
            nonzeroflux = tempflux[tminindex:tmaxindex + 1]
            newflux = (nonzeroflux - min(nonzeroflux)) / (max(nonzeroflux) - min(nonzeroflux))
            newflux2 = np.concatenate((tempflux[0:tminindex], newflux, tempflux[tmaxindex + 1:]))
            images = np.append(images, np.array([newflux2]), axis=0)  # images.append(newflux2)
            filenames.append(self.filename + "_" + str(-z))
            redshifts.append(-z)

        inputImages = np.array(images)
        inputFilenames = np.array(filenames)
        inputRedshifts = np.array(redshifts)

        return inputImages, inputFilenames, inputRedshifts, self.typeNamesList


    def saveArrays(self):
        inputImages, inputFilenames, inputRedshifts = self.redshifting()
        np.savez_compressed('input_data.npz', inputImages=inputImages, inputFilenames=inputFilenames,
                            inputRedshifts=inputRedshifts, typeNamesList = self.typeNamesList)
##
#sfTemplateLocation = '/home/dan/Desktop/SNClassifying_Pre-alpha/templates/superfit_templates/sne/'
##sfFilename = 'Ia/sn1981b.max.dat'
##
##filename = sfFilename
##
##with open('training_params.pickle') as f:
##    nTypes, w0, w1, nw, minAge, maxAge, ageBinSize = pickle.load(f)
##    
##minZ = 0
##maxZ = 0.5
##
##InputSpectra(filename, minZ, maxZ, nTypes, minAge, maxAge, ageBinSize, w0, w1, nw).saveArrays()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'design.ui'
#
# Created by: PyQt4 UI code generator 4.11.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.resize(1181, 853)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))
        self.horizontalLayout_3 = QtGui.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.verticalLayout = QtGui.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtGui.QLayout.SetMaximumSize)
        self.verticalLayout.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox_2 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout_7 = QtGui.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.gridLayout_2 = QtGui.QGridLayout()
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.btnBrowse = QtGui.QPushButton(self.groupBox_2)
        self.btnBrowse.setMinimumSize(QtCore.QSize(100, 0))
        self.btnBrowse.setMaximumSize(QtCore.QSize(100, 50))
        self.btnBrowse.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.btnBrowse.setObjectName(_fromUtf8("btnBrowse"))
        self.gridLayout_2.addWidget(self.btnBrowse, 1, 0, 1, 1, QtCore.Qt.AlignHCenter)
        self.horizontalLayout_8 = QtGui.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.lblInputFilename = QtGui.QLabel(self.groupBox_2)
        self.lblInputFilename.setMaximumSize(QtCore.QSize(200, 16777215))
        self.lblInputFilename.setObjectName(_fromUtf8("lblInputFilename"))
        self.horizontalLayout_8.addWidget(self.lblInputFilename, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter)
        self.gridLayout_2.addLayout(self.horizontalLayout_8, 0, 0, 1, 1)
        self.gridLayout_2.setRowStretch(0, 10)
        self.gridLayout_2.setRowStretch(1, 7)
        self.verticalLayout_7.addLayout(self.gridLayout_2)
        self.verticalLayout.addWidget(self.groupBox_2)
        self.groupBox = QtGui.QGroupBox(self.centralwidget)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_6 = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.scrollArea = QtGui.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName(_fromUtf8("scrollArea"))
        self.scrollAreaWidgetContents = QtGui.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 224, 252))
        self.scrollAreaWidgetContents.setObjectName(_fromUtf8("scrollAreaWidgetContents"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_4.setContentsMargins(-1, 0, -1, 0)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.checkBoxZeroZTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxZeroZTrained.setChecked(False)
        self.checkBoxZeroZTrained.setObjectName(_fromUtf8("checkBoxZeroZTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxZeroZTrained)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtGui.QSpacerItem(0, 0, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.checkBoxKnownZ = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxKnownZ.setObjectName(_fromUtf8("checkBoxKnownZ"))
        self.horizontalLayout.addWidget(self.checkBoxKnownZ)
        self.lineEditKnownZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditKnownZ.setObjectName(_fromUtf8("lineEditKnownZ"))
        self.horizontalLayout.addWidget(self.lineEditKnownZ)
        self.horizontalLayout.setStretch(2, 1)
        self.horizontalLayout.setStretch(3, 1)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.checkBoxAgnosticZTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxAgnosticZTrained.setCheckable(True)
        self.checkBoxAgnosticZTrained.setTristate(False)
        self.checkBoxAgnosticZTrained.setObjectName(_fromUtf8("checkBoxAgnosticZTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxAgnosticZTrained)
        self.checkBoxGalTrained = QtGui.QCheckBox(self.scrollAreaWidgetContents)
        self.checkBoxGalTrained.setObjectName(_fromUtf8("checkBoxGalTrained"))
        self.verticalLayout_3.addWidget(self.checkBoxGalTrained)
        self.verticalLayout_4.addLayout(self.verticalLayout_3)
        self.verticalLayout_5 = QtGui.QVBoxLayout()
        self.verticalLayout_5.setContentsMargins(-1, -1, -1, 0)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEditMinZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMinZ.setObjectName(_fromUtf8("lineEditMinZ"))
        self.gridLayout.addWidget(self.lineEditMinZ, 0, 1, 1, 1)
        self.label_2 = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.lineEditMaxZ = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditMaxZ.setObjectName(_fromUtf8("lineEditMaxZ"))
        self.gridLayout.addWidget(self.lineEditMaxZ, 1, 1, 1, 1)
        self.verticalLayout_5.addLayout(self.gridLayout)
        self.horizontalLayout_4 = QtGui.QHBoxLayout()
        self.horizontalLayout_4.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.label_3 = QtGui.QLabel(self.scrollAreaWidgetContents)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.horizontalLayout_4.addWidget(self.label_3)
        self.lineEditSmooth = QtGui.QLineEdit(self.scrollAreaWidgetContents)
        self.lineEditSmooth.setMaximumSize(QtCore.QSize(40, 16777215))
        self.lineEditSmooth.setObjectName(_fromUtf8("lineEditSmooth"))
        self.horizontalLayout_4.addWidget(self.lineEditSmooth)
        self.horizontalSliderSmooth = QtGui.QSlider(self.scrollAreaWidgetContents)
        self.horizontalSliderSmooth.setMaximum(20)
        self.horizontalSliderSmooth.setProperty("value", 7)
        self.horizontalSliderSmooth.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderSmooth.setObjectName(_fromUtf8("horizontalSliderSmooth"))
        self.horizontalLayout_4.addWidget(self.horizontalSliderSmooth)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        self.verticalLayout_4.addLayout(self.verticalLayout_5)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_6.addWidget(self.scrollArea)
        self.horizontalLayout_6 = QtGui.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        self.btnRefit = QtGui.QPushButton(self.groupBox)
        self.btnRefit.setMaximumSize(QtCore.QSize(150, 16777215))
        self.btnRefit.setObjectName(_fromUtf8("btnRefit"))
        self.horizontalLayout_6.addWidget(self.btnRefit)
        self.verticalLayout_6.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtGui.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.btnCancel = QtGui.QPushButton(self.groupBox)
        self.btnCancel.setMinimumSize(QtCore.QSize(0, 0))
        self.btnCancel.setMaximumSize(QtCore.QSize(100, 16777215))
        self.btnCancel.setObjectName(_fromUtf8("btnCancel"))
        self.horizontalLayout_7.addWidget(self.btnCancel)
        self.verticalLayout_6.addLayout(self.horizontalLayout_7)
        self.verticalLayout.addWidget(self.groupBox)
        self.progressBar = QtGui.QProgressBar(self.centralwidget)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName(_fromUtf8("progressBar"))
        self.verticalLayout.addWidget(self.progressBar)
        spacerItem2 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.verticalLayout.setStretch(0, 5)
        self.verticalLayout.setStretch(1, 16)
        self.verticalLayout.setStretch(3, 12)
        self.horizontalLayout_3.addLayout(self.verticalLayout)
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.verticalLayout_2.setContentsMargins(-1, -1, 0, -1)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.groupBox_3 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_3.setObjectName(_fromUtf8("groupBox_3"))
        self.verticalLayout_8 = QtGui.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        self.listWidget = QtGui.QListWidget(self.groupBox_3)
        self.listWidget.setObjectName(_fromUtf8("listWidget"))
        item = QtGui.QListWidgetItem()
        self.listWidget.addItem(item)
        self.verticalLayout_8.addWidget(self.listWidget)
        self.verticalLayout_2.addWidget(self.groupBox_3)
        self.groupBox_4 = QtGui.QGroupBox(self.centralwidget)
        self.groupBox_4.setObjectName(_fromUtf8("groupBox_4"))
        self.verticalLayout_9 = QtGui.QVBoxLayout(self.groupBox_4)
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.horizontalLayout_9 = QtGui.QHBoxLayout()
        self.horizontalLayout_9.setSizeConstraint(QtGui.QLayout.SetDefaultConstraint)
        self.horizontalLayout_9.setContentsMargins(-1, -1, -1, 5)
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.label_9 = QtGui.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName(_fromUtf8("label_9"))
        self.horizontalLayout_9.addWidget(self.label_9)
        self.comboBoxSNType = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxSNType.setEditable(False)
        self.comboBoxSNType.setObjectName(_fromUtf8("comboBoxSNType"))
        self.horizontalLayout_9.addWidget(self.comboBoxSNType)
        self.comboBoxAge = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxAge.setObjectName(_fromUtf8("comboBoxAge"))
        self.horizontalLayout_9.addWidget(self.comboBoxAge)
        self.comboBoxHost = QtGui.QComboBox(self.groupBox_4)
        self.comboBoxHost.setObjectName(_fromUtf8("comboBoxHost"))
        self.comboBoxHost.addItem(_fromUtf8(""))
        self.horizontalLayout_9.addWidget(self.comboBoxHost)
        self.horizontalLayout_9.setStretch(0, 3)
        self.horizontalLayout_9.setStretch(1, 100)
        self.horizontalLayout_9.setStretch(2, 100)
        self.horizontalLayout_9.setStretch(3, 100)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_2 = QtGui.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(-1, 3, -1, -1)
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.labelTemplateName = QtGui.QLabel(self.groupBox_4)
        self.labelTemplateName.setMinimumSize(QtCore.QSize(270, 0))
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setItalic(True)
        self.labelTemplateName.setFont(font)
        self.labelTemplateName.setObjectName(_fromUtf8("labelTemplateName"))
        self.horizontalLayout_2.addWidget(self.labelTemplateName)
        self.pushButtonLeftTemplate = QtGui.QPushButton(self.groupBox_4)
        self.pushButtonLeftTemplate.setObjectName(_fromUtf8("pushButtonLeftTemplate"))
        self.horizontalLayout_2.addWidget(self.pushButtonLeftTemplate)
        self.pushButtonRightTemplate = QtGui.QPushButton(self.groupBox_4)
        self.pushButtonRightTemplate.setObjectName(_fromUtf8("pushButtonRightTemplate"))
        self.horizontalLayout_2.addWidget(self.pushButtonRightTemplate)
        spacerItem3 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.label_4 = QtGui.QLabel(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.horizontalLayout_2.addWidget(self.label_4)
        self.lineEditRedshift = QtGui.QLineEdit(self.groupBox_4)
        self.lineEditRedshift.setObjectName(_fromUtf8("lineEditRedshift"))
        self.horizontalLayout_2.addWidget(self.lineEditRedshift)
        self.horizontalSliderRedshift = QtGui.QSlider(self.groupBox_4)
        self.horizontalSliderRedshift.setMaximum(10000)
        self.horizontalSliderRedshift.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderRedshift.setObjectName(_fromUtf8("horizontalSliderRedshift"))
        self.horizontalLayout_2.addWidget(self.horizontalSliderRedshift)
        self.horizontalLayout_2.setStretch(5, 1)
        self.horizontalLayout_2.setStretch(6, 6)
        self.verticalLayout_9.addLayout(self.horizontalLayout_2)
        self.graphicsView_2 = PlotWidget(self.groupBox_4)
        self.graphicsView_2.setObjectName(_fromUtf8("graphicsView_2"))
        self.verticalLayout_9.addWidget(self.graphicsView_2)
        self.graphicsView = PlotWidget(self.groupBox_4)
        self.graphicsView.setObjectName(_fromUtf8("graphicsView"))
        self.verticalLayout_9.addWidget(self.graphicsView)
        self.verticalLayout_9.setStretch(2, 1)
        self.verticalLayout_9.setStretch(3, 3)
        self.verticalLayout_2.addWidget(self.groupBox_4)
        self.verticalLayout_2.setStretch(0, 3)
        self.verticalLayout_2.setStretch(1, 10)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.horizontalLayout_3.setStretch(0, 9)
        self.horizontalLayout_3.setStretch(1, 32)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1181, 25))
        self.menubar.setObjectName(_fromUtf8("menubar"))
        MainWindow.setMenuBar(self.menubar)
        self.statusBar = QtGui.QStatusBar(MainWindow)
        self.statusBar.setObjectName(_fromUtf8("statusBar"))
        MainWindow.setStatusBar(self.statusBar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.groupBox_2.setTitle(_translate("MainWindow", "Select Spectra", None))
        self.btnBrowse.setText(_translate("MainWindow", "Browse", None))
        self.lblInputFilename.setText(_translate("MainWindow", "Select SN File...", None))
        self.groupBox.setTitle(_translate("MainWindow", "Priors", None))
        self.checkBoxZeroZTrained.setText(_translate("MainWindow", "Trained at z = 0", None))
        self.checkBoxKnownZ.setText(_translate("MainWindow", "Known z", None))
        self.checkBoxAgnosticZTrained.setText(_translate("MainWindow", "Agnostic Redshift Model", None))
        self.checkBoxGalTrained.setText(_translate("MainWindow", "SN + Gal Trained Model", None))
        self.label.setText(_translate("MainWindow", "Min Redshift", None))
        self.lineEditMinZ.setText(_translate("MainWindow", "0", None))
        self.label_2.setText(_translate("MainWindow", "Max Redshift", None))
        self.lineEditMaxZ.setText(_translate("MainWindow", "0.5", None))
        self.label_3.setText(_translate("MainWindow", "Smooth", None))
        self.lineEditSmooth.setText(_translate("MainWindow", "7", None))
        self.btnRefit.setText(_translate("MainWindow", "Re-fit with priors", None))
        self.btnCancel.setText(_translate("MainWindow", "Cancel", None))
        self.groupBox_3.setTitle(_translate("MainWindow", "Best Matches", None))
        __sortingEnabled = self.listWidget.isSortingEnabled()
        self.listWidget.setSortingEnabled(False)
        item = self.listWidget.item(0)
        item.setText(_translate("MainWindow", "Best Matches", None))
        self.listWidget.setSortingEnabled(__sortingEnabled)
        self.groupBox_4.setTitle(_translate("MainWindow", "Analyse selection", None))
        self.label_9.setText(_translate("MainWindow", "Plot Template", None))
        self.comboBoxHost.setItemText(0, _translate("MainWindow", "Host Galaxy", None))
        self.labelTemplateName.setText(_translate("MainWindow", "Template Name", None))
        self.pushButtonLeftTemplate.setText(_translate("MainWindow", "<", None))
        self.pushButtonRightTemplate.setText(_translate("MainWindow", ">", None))
        self.label_4.setText(_translate("MainWindow", "Redshift", None))
        self.lineEditRedshift.setText(_translate("MainWindow", "0", None))

from pyqtgraph import PlotWidget



import numpy as np
import itertools
import tensorflow as tf
from multilayer_convnet import convnet_variables
import os

scriptDirectory = os.path.dirname(os.path.abspath(__file__))
loaded = np.load(os.path.join(scriptDirectory, "type_age_atRedshiftZero.npz"))
trainImages = loaded['trainImages']
trainLabels = loaded['trainLabels']
# trainFilenames = loaded['trainFilenames']
# trainTypeNames = loaded['trainTypeNames']
testImages = loaded['testImages']
testLabels = loaded['testLabels']
# testFilenames = loaded['testFilenames']
testTypeNames = loaded['testTypeNames']
typeNamesList = loaded['typeNamesList']
# validateImages = sortData[2][0]
# validateLabels = sortData[2][1]


print("Completed creatingArrays")

N = 1024
ntypes = len(testLabels[0])
imWidth = 32  # Image size and width
imWidthReduc = 8

a = []

sess = tf.InteractiveSession()

x, y_, keep_prob, y_conv = convnet_variables(imWidth, imWidthReduc, N, ntypes)



# TRAIN AND EVALUATE MODEL
cross_entropy = tf.reduce_mean(-tf.reduce_sum(y_ * tf.log(y_conv), reduction_indices=[1]))
train_step = tf.train.AdamOptimizer(1e-4).minimize(cross_entropy)
correct_prediction = tf.equal(tf.argmax(y_conv, 1), tf.argmax(y_, 1))
accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))

sess.run(tf.initialize_all_variables())

trainImagesCycle = itertools.cycle(trainImages)
trainLabelsCycle = itertools.cycle(trainLabels)
for i in range(30000):
    batch_xs = np.array(list(itertools.islice(trainImagesCycle, 50 * i, 50 * i + 50)))
    batch_ys = np.array(list(itertools.islice(trainLabelsCycle, 50 * i, 50 * i + 50)))
    train_step.run(feed_dict={x: batch_xs, y_: batch_ys, keep_prob: 0.5})
    if (i % 100 == 0):
        train_accuracy = accuracy.eval(feed_dict={x: batch_xs, y_: batch_ys, keep_prob: 1.0})
        print("step %d, training accuracy %g" % (i, train_accuracy))
        testacc = accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
        print("test accuracy %g" % (testacc))
        a.append(testacc)
print("test accuracy %g" % accuracy.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0}))

yy = y_conv.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
cp = correct_prediction.eval(feed_dict={x: testImages, y_: testLabels, keep_prob: 1.0})
print(cp)
for i in range(len(cp)):
    if (cp[i] == False):
        predictedIndex = np.argmax(yy[i])
        print(i, testTypeNames[i], typeNamesList[predictedIndex])

# ACTUAL ACCURACY, SUBTYPE ACCURACY, AGE ACCURACY
typeAndAgeCorrect = 0
typeCorrect = 0
subTypeCorrect = 0
subTypeAndAgeCorrect = 0
typeAndNearAgeCorrect = 0
subTypeAndNearAgeCorrect = 0
for i in range(len(testTypeNames)):
    predictedIndex = np.argmax(yy[i])
    testSubType = testTypeNames[i][0:2]
    actualSubType = typeNamesList[predictedIndex][0:2]
    if testTypeNames[i][0:3] == 'IIb':
        testSubType = 'Ib'
    if typeNamesList[predictedIndex][0:3] == 'IIb':
        actualSubType = 'Ib'
    testType = testTypeNames[i].split(': ')[0]
    actualType = typeNamesList[predictedIndex].split(': ')[0]
    testAge = testTypeNames[i].split(': ')[1]
    actualAge = typeNamesList[predictedIndex].split(': ')[1]
    nearTestAge = testAge.split(' to ')

    if (testTypeNames[i] == typeNamesList[predictedIndex]):
        typeAndAgeCorrect += 1
    if (testType == actualType):  # correct type
        typeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (
            nearTestAge[1] in actualAge)):  # check if the age is in the neigbouring bin
            typeAndNearAgeCorrect += 1  # all correct except nearby bin
    if (testSubType == actualSubType):  # correct subtype
        subTypeCorrect += 1
        if testAge == actualAge:
            subTypeAndAgeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (
            nearTestAge[1] in actualAge)):  # check if the age is in the neigbouring bin
            subTypeAndNearAgeCorrect += 1  # subtype and nearby bin

typeAndAgeAccuracy = float(typeAndAgeCorrect) / len(testTypeNames)
typeAccuracy = float(typeCorrect) / len(testTypeNames)
subTypeAccuracy = float(subTypeCorrect) / len(testTypeNames)
subTypeAndAgeAccuracy = float(subTypeAndAgeCorrect) / len(testTypeNames)
typeAndNearAgeAccuracy = float(typeAndNearAgeCorrect) / len(testTypeNames)
subTypeAndNearAgeAccuracy = float(subTypeAndNearAgeCorrect) / len(testTypeNames)

print("typeAndAgeAccuracy : " + str(typeAndAgeAccuracy))
print("typeAccuracy : " + str(typeAccuracy))
print("subTypeAccuracy : " + str(subTypeAccuracy))
print("subTypeAndAgeAccuracy: " + str(subTypeAndAgeAccuracy))
print("typeAndNearAgeAccuracy : " + str(typeAndNearAgeAccuracy))
print("subTypeAndNearAgeAccuracy : " + str(subTypeAndNearAgeAccuracy))

# SAVE THE MODEL
saver = tf.train.Saver()
save_path = saver.save(sess, "model_trainedAtZeroZ.ckpt")
print("Model saved in file: %s" % save_path)



import os
import numpy as np

def save_templates():
    nTypes = len(trainLabels[0])

    templateFluxesAll = []
    templateLabelsAll = []
    templateNamesAll = []
    templateFilenamesAll = []

    for c in range(nTypes):
        templateFluxesForThisType = []
        templateLabelsForThisType = []
        templateNamesForThisType = []
        templateFilenamesForThisType = []
        countTemplates = 0
        for i in range(len(trainLabels)):
            if (trainLabels[i][c] == 1):
                templateFluxesForThisType.append(trainImages[i])
                templateLabelsForThisType.append(trainLabels[i])
                templateNamesForThisType.append(trainTypeNames[i])
                templateFilenamesForThisType.append(trainFilenames[i].replace('.lnw', '').strip('_z0.0'))
                countTemplates += 1
                
        if countTemplates == 0:
            templateFluxesForThisType.append(np.zeros(len(trainImages[0])))
            templateLabelsForThisType.append(np.zeros(len(trainLabels[0])))
            templateNamesForThisType.append('No Templates')
            templateFilenamesForThisType.append('No Templates')

        templateFluxesAll.append(np.array(templateFluxesForThisType))
        templateLabelsAll.append(np.array(templateLabelsForThisType))
        templateNamesAll.append(np.array(templateNamesForThisType))
        templateFilenamesAll.append(np.array(templateFilenamesForThisType))

    # These are nTypes by numOfTemplatesForEachType dimensional arrays. Each entry in this 2D array has a 1D flux
    # They are in order of nTypes
    templateFluxesAll = np.array(templateFluxesAll)
    templateLabelsAll = np.array(templateLabelsAll)
    templateNamesAll = np.array(templateNamesAll)
    templateFilenamesAll = np.array(templateFilenamesAll)
    
    np.savez_compressed(saveFilename, templateFluxesAll=templateFluxesAll, templateLabelsAll=templateLabelsAll, templateNamesAll=templateNamesAll, templateFilenamesAll=templateFilenamesAll)

if __name__ == '__main__':
    scriptDirectory = os.path.dirname(os.path.abspath(__file__))
    trainingSet = "type_age_atRedshiftZero.npz"
    print("Reading " + trainingSet + " ...")
    loaded = np.load(os.path.join(scriptDirectory, trainingSet))
    trainImages = loaded['trainImages']
    trainLabels = loaded['trainLabels']
    trainFilenames = loaded['trainFilenames']
    trainTypeNames = loaded['trainTypeNames']

    saveFilename = 'templates.npz'

    print("Saving Templates...")
    save_templates()
    print("Saved Templates to: " + saveFilename)

    loaded = np.load(os.path.join(scriptDirectory, saveFilename))



import matplotlib.pyplot as plt
import numpy as np
import itertools
import tensorflow as tf

loaded = np.load('type_age_atRedshiftZero.npz')
trainImages = loaded['trainImages']
trainLabels = loaded['trainLabels']
#trainFilenames = loaded['trainFilenames']
#trainTypeNames = loaded['trainTypeNames']
testImages = loaded['testImages']
testLabels = loaded['testLabels']
#testFilenames = loaded['testFilenames']
testTypeNames = loaded['testTypeNames']
typeNamesList = loaded['typeNamesList']
#validateImages = sortData[2][0]
#validateLabels = sortData[2][1]



print("Completed creatingArrays")

N = 1024
nBins = len(testLabels[0])
print(nBins)

a = []

#IMPLEMENTING THE REGRESSSION
x = tf.placeholder(tf.float32, [None, N])

W = tf.Variable(tf.zeros([N, nBins]))
b = tf.Variable(tf.zeros([nBins]))

y = tf.nn.softmax(tf.matmul(x, W) + b)


#TRAINING
y_ = tf.placeholder(tf.float32, [None, nBins]) #correct answers

cross_entropy = tf.reduce_mean(-tf.reduce_sum(y_ * tf.log(tf.clip_by_value(y,1e-10,1.0)), reduction_indices=[1]))

train_step = tf.train.GradientDescentOptimizer(0.5).minimize(cross_entropy)

init = tf.initialize_all_variables()

sess = tf.Session()
sess.run(init)

#batch_xs1 = trainImages
#batch_ys1 = trainLabels
#print(sess.run(y, feed_dict={x: batch_xs1, y_: batch_ys1}))

#Train 1000 times
trainImagesCycle = itertools.cycle(trainImages)
trainLabelsCycle = itertools.cycle(trainLabels)
for i in range(4000):
    batch_xs = np.array(list(itertools.islice(trainImagesCycle, 5000*i, 5000*i+5000)))
    batch_ys = np.array(list(itertools.islice(trainLabelsCycle, 5000*i, 5000*i+5000)))
    sess.run(train_step, feed_dict={x: batch_xs, y_: batch_ys})
    if (i % 100 == 1):
        correct_prediction = tf.equal(tf.argmax(y,1), tf.argmax(y_,1))
        accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))
        testacc = sess.run(accuracy, feed_dict={x: testImages, y_: testLabels})
        trainacc = sess.run(accuracy, feed_dict={x: trainImages[0:1000], y_: trainLabels[0:1000]})
        a.append(testacc)
        print(i, str(testacc) + " " + str(trainacc))

batch_xs1 = testImages
batch_ys1 = testLabels
#print(sess.run(y, feed_dict={x: batch_xs1, y_: batch_ys1}))
yy = sess.run(y, feed_dict={x: testImages, y_: testLabels})

#EVALUATING THE MODEL
correct_prediction = tf.equal(tf.argmax(y,1), tf.argmax(y_,1))
accuracy = tf.reduce_mean(tf.cast(correct_prediction, tf.float32))
print(sess.run(accuracy, feed_dict={x: testImages, y_: testLabels}))


yy = sess.run(y, feed_dict={x: testImages, y_: testLabels})
cp = sess.run(correct_prediction, feed_dict={x: testImages, y_: testLabels})
print(cp)
for i in range(len(cp)):
    if (cp[i] == False):
        predictedIndex = np.argmax(yy[i])
        print(i, testTypeNames[i], typeNamesList[predictedIndex])


#ACTUAL ACCURACY, SUBTYPE ACCURACY, AGE ACCURACY
typeAndAgeCorrect = 0
typeCorrect = 0
subTypeCorrect = 0
subTypeAndAgeCorrect = 0
typeAndNearAgeCorrect = 0
subTypeAndNearAgeCorrect = 0
for i in range(len(testTypeNames)):
    predictedIndex = np.argmax(yy[i])
    testSubType = testTypeNames[i][0:2]
    actualSubType = typeNamesList[predictedIndex][0:2]
    testType = testTypeNames[i].split(': ')[0]
    actualType = typeNamesList[predictedIndex].split(': ')[0]
    testAge = testTypeNames[i].split(': ')[1]
    actualAge = typeNamesList[predictedIndex].split(': ')[1]
    nearTestAge = testAge.split(' to ')
    
    if (testTypeNames[i] == typeNamesList[predictedIndex]):
        typeAndAgeCorrect += 1
    if (testType == actualType): #correct type
        typeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge)): #check if the age is in the neigbouring bin
            typeAndNearAgeCorrect += 1 #all correct except nearby bin
    if (testSubType == actualSubType): #correct subtype
        subTypeCorrect += 1
        if testAge == actualAge:
            subTypeAndAgeCorrect += 1
        if ((nearTestAge[0] in actualAge) or (nearTestAge[1] in actualAge)): #check if the age is in the neigbouring bin
            subTypeAndNearAgeCorrect += 1 #subtype and nearby bin

typeAndAgeAccuracy = float(typeAndAgeCorrect)/len(testTypeNames)
typeAccuracy = float(typeCorrect)/len(testTypeNames)
subTypeAccuracy = float(subTypeCorrect)/len(testTypeNames)
subTypeAndAgeAccuracy = float(subTypeAndAgeCorrect)/len(testTypeNames)
typeAndNearAgeAccuracy = float(typeAndNearAgeCorrect)/len(testTypeNames)
subTypeAndNearAgeAccuracy = float(subTypeAndNearAgeCorrect)/len(testTypeNames)

print("typeAndAgeAccuracy : " + str(typeAndAgeAccuracy))
print("typeAccuracy : " + str(typeAccuracy))
print("subTypeAccuracy : " + str(subTypeAccuracy))
print("subTypeAndAgeAccuracy: " + str(subTypeAndAgeAccuracy))
print("typeAndNearAgeAccuracy : " + str(typeAndNearAgeAccuracy))
print("subTypeAndNearAgeAccuracy : " + str(subTypeAndNearAgeAccuracy))
    

#SAVE THE MODEL
saver = tf.train.Saver()
save_path = saver.save(sess, "model.ckpt")
print("Model saved in file: %s" % save_path)





import sys
import os
mainDirectory = os.path.dirname(os.path.abspath(__file__))

# try:
#     from PyQt4 import QtGui
# except ImportError:
#     from main import *
#     print "Warning: You will need to install 'PyQt4' if you want to use the graphical interface. " \
#           "Using the automatic library will continue to work."

from classify import Classify


def main():
    classification = Classify(filenames=['test_spectrum_file.dat',
                                         'test_spectrum_file.dat'],
                              redshifts=[0.34, 0.13])
    print classification.list_best_matches()
    # classification.plot_with_gui(indexToPlot=1)


def run_gui():
    app = QtGui.QApplication(sys.argv)
    form = MainApp()
    form.show()
    app.exec_()


if __name__ == '__main__':
    main()
    # run_gui()





import tensorflow as tf
import numpy as np
import pickle
from input_spectra import *
from multilayer_convnet import convnet_variables
import os
import urllib

scriptDirectory = os.path.dirname(os.path.abspath(__file__))

print scriptDirectory
modelFilename = os.path.join(scriptDirectory, 'model_trainedAtZeroZ.ckpt')
if not os.path.isfile(modelFilename):
    print "Downloading Training Model..."
    modelFileDownload = urllib.URLopener()
    modelFileDownload.retrieve(
        "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/model_trainedAtZeroZ.ckpt", modelFilename)
    print modelFilename
dataFilename = os.path.join(scriptDirectory, 'type_age_atRedshiftZero.npz')
if not os.path.isfile(dataFilename):
    print "Downloading Data File..."
    dataFileDownload = urllib.URLopener()
    dataFileDownload.retrieve(
        "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/type_age_atRedshiftZero.npz", dataFilename)
    print dataFilename
dataFilename = os.path.join(scriptDirectory, 'training_params.pickle')
if not os.path.isfile(dataFilename):
    print "Downloading Data File..."
    dataFileDownload = urllib.URLopener()
    dataFileDownload.retrieve(
        "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/training_params.pickle", dataFilename)
    print dataFilename
dataFilename = os.path.join(scriptDirectory, 'templates.npz')
if not os.path.isfile(dataFilename):
    print "Downloading Data File..."
    dataFileDownload = urllib.URLopener()
    dataFileDownload.retrieve(
        "https://raw.githubusercontent.com/daniel-muthukrishna/DASH/master/dash/templates.npz", dataFilename)
    print dataFilename


loaded = np.load(os.path.join(scriptDirectory, "type_age_atRedshiftZero.npz"))
trainImages = loaded['trainImages']
trainLabels = loaded['trainLabels']
# loaded = np.load(os.path.join(scriptDirectory, "templates.npz"))
# trainImages = loaded['templateFluxesAll']
# trainLabels = loaded['templateLabelsAll']
##trainFilenames = loaded['trainFilenames']
##trainTypeNames = loaded['trainTypeNames']
##testImages = loaded['testImages']
##testLabels = loaded['testLabels']
##testFilenames = loaded['testFilenames']
##testTypeNames = loaded['testTypeNames']
##typeNamesList = loaded['typeNamesList']


class LoadInputSpectra(object):
    def __init__(self, inputFilename, minZ, maxZ, smooth):
        with open(os.path.join(scriptDirectory, "training_params.pickle")) as f:
            nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList = pickle.load(f)

        self.nw = nw
        self.nTypes = nTypes
        
        self.inputSpectra = InputSpectra(inputFilename, minZ, maxZ, nTypes, minAge, maxAge, ageBinSize, w0, w1, nw, typeList, smooth)

        self.inputImages, self.inputFilenames, self.inputRedshifts, self.typeNamesList = self.inputSpectra.redshifting()
        self.nBins = len(self.typeNamesList)

    def input_spectra(self):
        return self.inputImages, self.inputRedshifts, self.typeNamesList, int(self.nw), self.nBins


class RestoreModel(object):
    def __init__(self, modelFilename, inputImages, nw, nBins):
        self.reset()

        self.modelFilename = modelFilename
        self.inputImages = inputImages
        self.nw = nw
        self.nBins = nBins
        self.imWidthReduc = 8
        self.imWidth = 32 #Image size and width

        self.x, self.y_, self.keep_prob, self.y_conv = convnet_variables(self.imWidth, self.imWidthReduc, self.nw, self.nBins)

        self.saver = tf.train.Saver()

    def restore_variables(self):
        with tf.Session() as sess:
            self.saver.restore(sess, self.modelFilename)

            softmax = self.y_conv.eval(feed_dict={self.x: self.inputImages, self.keep_prob: 1.0})
            # print(softmax)
            
        return softmax

    def reset(self):
        tf.reset_default_graph()

class BestTypesListSingleRedshift(object):
    def __init__(self, modelFilename, inputImage, typeNamesList, nw, nBins):
        self.modelFilename = modelFilename
        self.inputImage = inputImage
        self.typeNamesList = typeNamesList
        self.nBins = nBins

        self.restoreModel = RestoreModel(self.modelFilename, self.inputImage, nw, nBins)
        self.typeNamesList = np.array(typeNamesList)
        self.inputImage = self.inputImage[0]
        self.softmax = self.restoreModel.restore_variables()[0]
        # print(len(self.softmax))

        self.bestTypes, self.idx, self.softmaxOrdered = self.create_list()


    def create_list(self):
        idx = np.argsort(self.softmax) #list of the index of the highest probabiites
        bestTypes = self.typeNamesList[idx[::-1]] #reordered in terms of softmax probability columns

        return bestTypes, idx, self.softmax[idx[::-1]]

    def plot_best_types(self):
        inputFluxes = []
        templateFluxes = []
        for j in range(20):#len(self.bestTypes)): #index of best Types in order
            c = self.idx[::-1][j]
            typeName = self.typeNamesList[c] #Name of best type
            for i in range(len(trainLabels)): #Checking through templates
                if (trainLabels[i][c] == 1):    #to find template for the best Type
                    templateFlux = trainImages[i]  #plot template
                    inputFlux = self.inputImage #Plot inputImage at red
                    break
            if (i == len(trainLabels)-1):
                print("No Template") #NEED TO GET TEMPLATE PLOTS IN A BETTER WAY
                templateFlux = np.zeros(len(trainImages[0]))
                inputFlux = self.inputImage

            templateFluxes.append(templateFlux)
            inputFluxes.append(inputFlux)
            
        templateFluxes = np.array(templateFluxes)
        inputFluxes = np.array(inputFluxes)
        return templateFluxes, inputFluxes

    

class BestTypesList(object):
    def __init__(self, modelFilename, inputImages, inputRedshifts, typeNamesList, nw, nBins):
        self.modelFilename = modelFilename
        self.inputImages = inputImages
        self.inputRedshifts = inputRedshifts
        self.typeNamesList = typeNamesList
        self.nBins = nBins
        
        self.restoreModel = RestoreModel(self.modelFilename, self.inputImages, nw, nBins)
        self.yInputRedshift = self.restoreModel.restore_variables() #softmax at each redshift

        self.bestForEachType, self.redshiftIndex = self.create_list()

    def create_list(self):
        bestForEachType = np.zeros((self.nBins,3))
        redshiftIndex = np.zeros(self.nBins)        #best redshift index for each type
        for i in range(len(self.inputRedshifts)): #for each redshift
            z = self.inputRedshifts[i]       #redshift
            softmax = self.yInputRedshift[i] #softmax probabilities at particular redshift
            bestIndex = np.argmax(softmax) #index of the Best type at redshift
            if softmax[bestIndex] > bestForEachType[bestIndex][2]: #if relProb of best type at this redshift is better than the relProb of this type at any redshift
                bestForEachType[bestIndex][2] = softmax[bestIndex]
                bestForEachType[bestIndex][1] = z
                bestForEachType[bestIndex][0] = bestIndex
                redshiftIndex[bestIndex] = i
        idx = np.argsort(bestForEachType[:,2]) #list of the index of the highest probabiites
        bestForEachType = bestForEachType[idx[::-1]] #reordered in terms of relProb columns
        
        return bestForEachType, redshiftIndex

    def print_list(self):
        return self.bestForEachType, self.redshiftIndex

    def plot_best_types(self):
        #bestForEachType, typeNamesList, redshiftIndex = self.create_list()
        inputFluxes = []
        templateFluxes = []
        for j in range(len(self.bestForEachType)): #index of best Types in order
            c = int(self.bestForEachType[:,0][j])
            typeName = self.typeNamesList[c] #Name of best type
            for i in range(0,len(trainLabels)): #Checking through templates
                if (trainLabels[i][c] == 1):    #to find template for the best Type
                    templateFlux = trainImages[i]  #plot template
                    inputFlux = self.inputImages[int(self.redshiftIndex[c])] #Pliot inputImage at red
                    #plt.title(typeName+ ": " + str(bestForEachType[c][1]))
                    break
            if (i == len(trainLabels)-1):
                #print("No Template") #NEED TO GET TEMPLATE PLOTS IN A BETTER WAY
                templateFlux = np.zeros(len(trainImages[0]))
                inputFlux = self.inputImages[int(self.redshiftIndex[c])]

            templateFluxes.append(templateFlux)
            inputFluxes.append(inputFlux)
            
        templateFluxes = np.array(templateFluxes)
        inputFluxes = np.array(inputFluxes)
        return templateFluxes, inputFluxes

    def redshift_graph(self):
        #bestForEachType, typeNamesList, redshiftIndex = self.create_list()
        self.redshiftGraphs = []#redshiftGraphs = [[[],[]] for i in range(ntypes)]
        for j in range(len(self.bestForEachType)): #[0:2] takes top 2 entries
            c = int(self.bestForEachType[:,0][j])
            typeName = self.typeNamesList[c]
            #redshiftGraphs[c][0] = self.inputRedshifts
            self.redshiftGraphs.append(self.yInputRedshift[:,c])
            

        self.redshiftGraphs = np.array(self.redshiftGraphs)

        return self.inputRedshifts, self.redshiftGraphs

#bestTypesList = BestTypesList("/tmp/model.ckpt")
#templateFluxes, inputFluxes = bestTypesList.plot_best_types()

##restoreModel = RestoreModel("/tmp/model.ckpt")
##yInputRedshift = restoreModel.restore_variables()
##
##
###Create List of Best Types
##bestForEachType = np.zeros((ntypes,3))
##redshiftIndex = np.zeros(ntypes) #best redshift index for each type
##for i in range(len(yInputRedshift)):
##    prob = yInputRedshift[i]
##    z = inputRedshifts[i]
##    bestIndex = np.argmax(prob)
##    if prob[bestIndex] > bestForEachType[bestIndex][2]:
##        bestForEachType[bestIndex][2] = prob[bestIndex]
##        bestForEachType[bestIndex][1] = z
##        bestForEachType[bestIndex][0] = bestIndex #inputTypeNames
##        redshiftIndex[bestIndex] = i
##
##idx = np.argsort(bestForEachType[:,2])
##bestForEachType = bestForEachType[idx[::-1]]
##
##print ("Type          Redshift      Rel. Prob.")
##print(bestForEachType)
##for i in range(10):#ntypes):
##    bestIndex = bestForEachType[i][0]
##    name, age = typeNamesList[bestIndex].split(': ')
##    print "".join(word.ljust(15) for word in [name, age , str(bestForEachType[i][1]), str(bestForEachType[i][2])])
##    #print(typeNamesList[bestIndex] + '\t' + str(bestForEachType[i][1]) + '\t' + str(bestForEachType[i][2]))
##
##
##
###Plot Each Best Type at corresponding best redshift
##for j in range(2):#len(bestForEachType)): #[0:2] takes top 2 entries
##    c = int(bestForEachType[:,0][j])
##    typeName = typeNamesList[c]
##    for i in range(0,len(trainImages)):
##        if (trainLabels[i][c] == 1):
##            print(i)
##            plt.plot(trainImages[i])
##            plt.plot(inputImages[redshiftIndex[c]])
##            plt.title(typeName+ ": " + str(bestForEachType[j][1]))
##            plt.show()
##            break
##        
##
###Plot Probability vs redshift for each class
##redshiftGraphs = [[[],[]] for i in range(ntypes)]
##for j in range(2):#len(bestForEachType)): #[0:2] takes top 2 entries
##    c = int(bestForEachType[:,0][j])
##    typeName = typeNamesList[c]
##    redshiftGraphs[c][0] = inputRedshifts
##    redshiftGraphs[c][1] = yInputRedshift[:,c]
##    plt.plot(redshiftGraphs[c][0],redshiftGraphs[c][1])
##    plt.xlabel("z")
##    plt.ylabel("Probability")
##    bestIndex = bestForEachType[c][0]
##    plt.title("Type: " + typeNamesList[bestIndex])
##    plt.show()
##
##redshiftGraphs = np.array(redshiftGraphs)



import pickle


typeList = ['Ia-norm', 'Ia-91T', 'Ia-91bg', 'Ia-csm', 'Ia-02cx', 'Ia-pec',
            'Ib-norm', 'Ibn', 'IIb', 'Ib-pec', 'Ic-norm', 'Ic-broad',
            'Ic-pec', 'IIP', 'IIL', 'IIn', 'II-pec']

nTypes = len(typeList)
w0 = 2500. #wavelength range in Angstroms
w1 = 10000.
nw = 1024. #number of wavelength bins
minAge = -20.
maxAge = 50.
ageBinSize = 4.


# Saving the objects:
with open('training_params.pickle', 'w') as f:
    pickle.dump([nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList], f)

# Getting back the objects:
with open('training_params.pickle') as f:
    nTypes, w0, w1, nw, minAge, maxAge, ageBinSize, typeList = pickle.load(f)



import os
import sys

from PyQt4 import QtGui
from PyQt4.QtCore import QThread, SIGNAL

import design

mainDirectory = os.path.dirname(os.path.abspath(__file__))

from dash.restore_model import *
from dash.create_arrays import AgeBinning

class MainApp(QtGui.QMainWindow, design.Ui_MainWindow):
    def __init__(self, parent=None, inputFilename="DefaultFilename"):
        super(MainApp, self).__init__(parent)
        self.setupUi(self)

        self.templates()
        self.plotted = False
        self.indexToPlot = 0
        self.plotZ = 0
        self.templateFluxes = np.zeros((2, int(self.nw)))
        self.inputFluxes = np.zeros((2, int(self.nw)))
        self.inputImageUnRedshifted = np.zeros((2, int(self.nw)))
        self.templatePlotFlux = np.zeros(int(self.nw))
        self.templateSubIndex = 0

        self.mainDirectory = os.path.dirname(os.path.abspath(__file__))

        self.pushButtonLeftTemplate.clicked.connect(self.select_sub_template_left)
        self.pushButtonRightTemplate.clicked.connect(self.select_sub_template_right)
        self.btnBrowse.clicked.connect(self.select_input_file)
        self.listWidget.itemClicked.connect(self.list_item_clicked)
        self.btnRefit.clicked.connect(self.fit_spectra)
        self.inputFilename = inputFilename
        self.progressBar.setValue(100)
        self.add_combo_box_entries()

        self.checkBoxZeroZTrained.stateChanged.connect(self.zero_redshift_model)
        self.checkBoxKnownZ.stateChanged.connect(self.zero_redshift_model)
        self.checkBoxAgnosticZTrained.stateChanged.connect(self.agnostic_redshift_model)

        self.checkBoxZeroZTrained.setChecked(True)
        self.checkBoxKnownZ.setChecked(True)
        self.checkBoxAgnosticZTrained.setEnabled(False)
        self.checkBoxGalTrained.setEnabled(False)
        self.comboBoxHost.setEnabled(False)

        self.horizontalSliderSmooth.valueChanged.connect(self.smooth_slider_changed)
        self.lineEditSmooth.textChanged.connect(self.smooth_text_changed)

        self.horizontalSliderRedshift.valueChanged.connect(self.redshift_slider_changed)
        self.lineEditRedshift.textChanged.connect(self.redshift_text_changed)

        self.comboBoxSNType.currentIndexChanged.connect(self.combo_box_changed)
        self.comboBoxAge.currentIndexChanged.connect(self.combo_box_changed)

    def templates(self):
        with open(os.path.join(mainDirectory, "training_params.pickle")) as f:
            self.nTypes, self.w0, self.w1, self.nw, minAge, maxAge, ageBinSize, self.typeList = pickle.load(f)

        dwlog = np.log(self.w1/self.w0)/self.nw
        self.wave = self.w0 * np.exp(np.arange(0,self.nw) * dwlog)

        loaded = np.load(os.path.join(mainDirectory, 'templates.npz'))
        self.templateFluxesAll = loaded['templateFluxesAll']
        self.templateFileNamesAll = loaded['templateFilenamesAll']

    def select_sub_template_right(self):
        self.templateSubIndex += 1
        self.plot_sub_templates()

    def select_sub_template_left(self):
        self.templateSubIndex -= 1
        self.plot_sub_templates()


    def plot_sub_templates(self):
        numOfSubTemplates = len(self.templateFileNamesAll[self.templateIndex])
        if self.templateSubIndex >= numOfSubTemplates:
            self.templateSubIndex = 0
        if self.templateSubIndex < 0:
            self.templateSubIndex = numOfSubTemplates - 1

        self.templatePlotFlux = self.templateFluxesAll[self.templateIndex][self.templateSubIndex]
        self.templatePlotName = self.templateFileNamesAll[self.templateIndex][self.templateSubIndex]
        print(self.templatePlotName)
        self.plot_best_matches()

    def combo_box_changed(self):
        self.templateIndex = self.comboBoxSNType.currentIndex() * (self.nTypes + 1) +  self.comboBoxAge.currentIndex()

        self.templatePlotFlux = self.templateFluxesAll[self.templateIndex][0]
        self.templatePlotName = self.templateFileNamesAll[self.templateIndex][0]
        print(self.templatePlotName)
        self.plot_best_matches()



    def add_combo_box_entries(self):
        minAge = -20
        maxAge = 50
        ageBinSize = 4
        ageLabels = AgeBinning(minAge, maxAge, ageBinSize).age_labels()
        for i in range(len(ageLabels)):
            self.comboBoxAge.addItem(ageLabels[i])

        for typeName in self.typeList:
            self.comboBoxSNType.addItem(typeName)


    def redshift_slider_changed(self):
        self.plotZ = self.horizontalSliderRedshift.value()/10000.
        self.lineEditRedshift.setText(str(self.plotZ))


    def redshift_text_changed(self):
        try:
            self.plotZ = float(self.lineEditRedshift.text())
            self.horizontalSliderRedshift.setValue(int(self.plotZ*10000))
            self.plot_best_matches()
        except ValueError:
            print("Redshift Value Error")

    def set_plot_redshift(self, plotZ):
        self.plotZ = plotZ
        self.lineEditRedshift.setText(str(plotZ))
        self.horizontalSliderRedshift.setValue(int(plotZ*10000))
        self.plot_best_matches()

    def smooth_slider_changed(self):
        self.lineEditSmooth.setText(str(self.horizontalSliderSmooth.value()))

    def smooth_text_changed(self):
        try:
            self.horizontalSliderSmooth.setValue(int(self.lineEditSmooth.text()))
        except ValueError:
            pass

    def zero_redshift_model(self):
        if (self.checkBoxZeroZTrained.isChecked() == True):
            self.modelFilename = os.path.join(self.mainDirectory, "model_trainedAtZeroZ.ckpt")
            self.checkBoxKnownZ.setEnabled(True)
            self.lineEditKnownZ.setEnabled(True)
            self.checkBoxAgnosticZTrained.setEnabled(False)
            self.checkBoxAgnosticZTrained.setChecked(False)

            if (self.checkBoxKnownZ.isChecked() == True):
                self.lineEditMinZ.setEnabled(False)
                self.lineEditMaxZ.setEnabled(False)
            else:
                self.lineEditMinZ.setEnabled(True)
                self.lineEditMaxZ.setEnabled(True)

        elif (self.checkBoxZeroZTrained.isChecked() == False):
            self.checkBoxKnownZ.setEnabled(False)
            self.lineEditKnownZ.setEnabled(False)
            self.checkBoxZeroZTrained.setEnabled(False)
            self.checkBoxZeroZTrained.setChecked(False)
            self.checkBoxAgnosticZTrained.setEnabled(True)
            self.checkBoxAgnosticZTrained.setChecked(True)
            self.agnostic_redshift_model()

    def agnostic_redshift_model(self):
        if (self.checkBoxAgnosticZTrained.isChecked() == True):
            self.checkBoxZeroZTrained.setEnabled(False)
            self.checkBoxZeroZTrained.setChecked(False)
            self.modelFilename = os.path.join(self.mainDirectory, "model_agnostic_redshift.ckpt")
        elif (self.checkBoxAgnosticZTrained.isChecked() == False):
            self.checkBoxZeroZTrained.setEnabled(True)
            self.checkBoxZeroZTrained.setChecked(True)
            self.zero_redshift_model()




        
    def select_input_file(self):
        inputFilename = QtGui.QFileDialog.getOpenFileName(self,"Select a spectrum file")
        print(inputFilename)
        print(self.inputFilename)
        if (inputFilename == self.inputFilename) or (inputFilename == ""):
            pass
        else:
            self.inputFilename = inputFilename
            self.lblInputFilename.setText(inputFilename.split('/')[-1])

            #Run InputSpectra

            self.fit_spectra()

    def fit_spectra(self):
        self.cancelledFitting = False
        self.progressBar.setMaximum(100) #
        self.progressBar.setValue(36)
        try:
            self.smooth = int(self.lineEditSmooth.text())
        except ValueError:
            QtGui.QMessageBox.critical(self, "Error", "Smooth must be positive integer")

        if (self.checkBoxKnownZ.isChecked() == True):
            self.redshiftFlag = True
            try:
                self.knownZ = float(self.lineEditKnownZ.text())
            except ValueError:
                QtGui.QMessageBox.critical(self, "Error", "Enter Known Redshift")
            self.minZ = self.knownZ
            self.maxZ = self.knownZ
            self.set_plot_redshift(self.knownZ)

            self.fitThread = FitSpectrumThread(self.inputFilename, self.minZ, self.maxZ, self.redshiftFlag, self.modelFilename, self.smooth)
            self.connect(self.fitThread, SIGNAL("load_spectrum_single_redshift(PyQt_PyObject)"), self.load_spectrum_single_redshift)
            self.connect(self.fitThread, SIGNAL("finished()"), self.done_fit_thread_single_redshift)
        else:
            self.redshiftFlag = False
            self.minZ = float(self.lineEditMinZ.text())
            self.maxZ = float(self.lineEditMaxZ.text())
            self.knownZ = self.minZ
            print (self.minZ, self.maxZ)
            self.fitThread = FitSpectrumThread(self.inputFilename, self.minZ, self.maxZ, self.redshiftFlag, self.modelFilename, self.smooth)
            self.connect(self.fitThread, SIGNAL("load_spectrum(PyQt_PyObject)"), self.load_spectrum)
            self.connect(self.fitThread, SIGNAL("finished()"), self.done_fit_thread)

        self.fitThread.start()

        self.btnCancel.clicked.connect(self.cancel)

    def cancel(self):
        if (self.cancelledFitting == False):
            self.cancelledFitting = True
            self.fitThread.terminate()
            self.progressBar.setValue(100)
            QtGui.QMessageBox.information(self, "Cancelled!", "Stopped Fitting Input Spectrum")


    def load_spectrum_single_redshift(self, spectrumInfo):
        self.bestTypes, self.softmax, self.idx, self.templateFluxes, self.inputFluxes, self.typeNamesList, self.inputImageUnRedshifted = spectrumInfo
        self.progressBar.setValue(85)#self.progressBar.value()+)
        print "here"

    def done_fit_thread_single_redshift(self):
        if (self.cancelledFitting == False):
            self.plotted = True
            self.list_best_matches_single_redshift()
            self.plot_best_matches()
            self.progressBar.setValue(100)
            QtGui.QMessageBox.information(self, "Done!", "Finished Fitting Input Spectrum")

    def list_best_matches_single_redshift(self):
        print("listing best matches...")
        self.listWidget.clear()
        self.listWidget.addItem("".join(word.ljust(25) for word in ['No.', 'Type', 'Age', 'Softmax Prob..']))
        for i in range(20):
            name, age = self.bestTypes[i].split(': ')
            self.listWidget.addItem("".join(word.ljust(25) for word in [str(i+1), name, age, str(self.softmax[i])]))

            if i == 0:
                SNTypeComboBoxIndex = self.comboBoxSNType.findText(name)
                self.comboBoxSNType.setCurrentIndex(SNTypeComboBoxIndex)
                AgeComboBoxIndex = self.comboBoxAge.findText(age)
                self.comboBoxAge.setCurrentIndex(AgeComboBoxIndex)

    def load_spectrum(self, spectrumInfo):
        self.bestForEachType, self.templateFluxes, self.inputFluxes, self.inputRedshifts, self.redshiftGraphs, self.typeNamesList = spectrumInfo
        self.progressBar.setValue(85)#self.progressBar.value()+)    

    def done_fit_thread(self):
        if (self.cancelledFitting == False):
            self.list_best_matches()
            self.plot_best_matches()
            self.plot_redshift_graphs()
            self.progressBar.setValue(100)
            QtGui.QMessageBox.information(self, "Done!", "Finished Fitting Input Spectrum")
        

    def list_best_matches(self):
        print("listing best matches...")
        self.listWidget.clear()
        self.listWidget.addItem("".join(word.ljust(25) for word in ['No.', 'Type', 'Age', 'Redshift', 'Rel. Prob.']))
        for i in range(20): #len(bestForEachType)
            bestIndex = int(self.bestForEachType[i][0])
            name, age = self.typeNamesList[bestIndex].split(': ')
            self.listWidget.addItem("".join(word.ljust(25) for word in [str(i+1), name, age , str(self.bestForEachType[i][1]), str(self.bestForEachType[i][2])]))


    def list_item_clicked(self, item):
        index, self.SNTypePlot, age1, age2, age3, softmax = str(item.text()).split()
        self.AgePlot = age1 + ' to ' + age3

        try:
            self.indexToPlot = int(index) - 1 #Two digit numbers
        except ValueError:
            self.indexToPlot = 0

        SNTypeComboBoxIndex = self.comboBoxSNType.findText(self.SNTypePlot)
        self.comboBoxSNType.setCurrentIndex(SNTypeComboBoxIndex)
        AgeComboBoxIndex = self.comboBoxAge.findText(self.AgePlot)
        self.comboBoxAge.setCurrentIndex(AgeComboBoxIndex)

        if self.redshiftFlag == False:
            self.plot_redshift_graphs()

        
    def plot_best_matches(self):
        if self.plotted == True:
            templateWave = self.wave * (1 + (self.plotZ))
            self.labelTemplateName.setText(self.templatePlotName)

            self.graphicsView.clear()

            if self.redshiftFlag == True:
                inputPlotFlux = self.inputImageUnRedshifted[0]
            elif self.redshiftFlag == False:
                inputPlotFlux = self.inputFluxes[self.indexToPlot]

            self.graphicsView.plot(self.wave, inputPlotFlux, name='Input Spectrum', pen={'color': (0, 255, 0)})
            self.graphicsView.plot(templateWave, self.templatePlotFlux, name=self.templatePlotName, pen={'color': (255,0,0)})
            self.graphicsView.setRange(xRange=[2500,10000])




    def plot_redshift_graphs(self):
        print("listing Redshift Graphs...")
        print(len(self.inputRedshifts), len(self.redshiftGraphs[self.indexToPlot]))
        self.graphicsView_2.clear()
        self.graphicsView_2.plot(self.inputRedshifts, self.redshiftGraphs[self.indexToPlot])
        self.graphicsView_2.setLabels(left=("Rel. Prob."), bottom=("z"))
        
    def browse_folder(self):
        self.listWidget.clear()
        directory = QtGui.QFileDialog.getExistingDirectory(self,"Pick a folder")

        if directory:
            for file_name in os.listdir(directory):
                self.listWidget.addItem(file_name)

    
class FitSpectrumThread(QThread):
    def __init__(self, inputFilename, minZ, maxZ, redshiftFlag, modelFilename, smooth):
        QThread.__init__(self)
        self.inputFilename = str(inputFilename)
        self.minZ = minZ
        self.maxZ = maxZ
        self.redshiftFlag = redshiftFlag
        self.modelFilename = modelFilename
        self.smooth = smooth

    def __del__(self):
        self.wait()

    def _input_spectrum(self):
        loadInputSpectra = LoadInputSpectra(self.inputFilename, self.minZ, self.maxZ, self.smooth)
        inputImages, inputRedshifts, typeNamesList, nw, nTypes = loadInputSpectra.input_spectra()
        bestTypesList = BestTypesList(self.modelFilename, inputImages, inputRedshifts, typeNamesList, nw, nTypes)
        bestForEachType, redshiftIndex = bestTypesList.print_list()
        templateFluxes, inputFluxes = bestTypesList.plot_best_types()
        inputRedshifts, redshiftGraphs = bestTypesList.redshift_graph()

        return (bestForEachType, templateFluxes, inputFluxes,
                inputRedshifts, redshiftGraphs, typeNamesList)

    def _input_spectrum_single_redshift(self):
        loadInputSpectraUnRedshifted = LoadInputSpectra(self.inputFilename, 0, 0, self.smooth)
        inputImageUnRedshifted, inputRedshift, typeNamesList, nw, nBins = loadInputSpectraUnRedshifted.input_spectra()

        loadInputSpectra = LoadInputSpectra(self.inputFilename, self.minZ, self.maxZ, self.smooth)
        inputImage, inputRedshift, typeNamesList, nw, nBins = loadInputSpectra.input_spectra()
        bestTypesList = BestTypesListSingleRedshift(self.modelFilename, inputImage, typeNamesList, nw, nBins)
        bestTypes = bestTypesList.bestTypes
        softmax = bestTypesList.softmaxOrdered
        idx = bestTypesList.idx
        templateFluxes, inputFluxes = bestTypesList.plot_best_types()

        return bestTypes, softmax, idx, templateFluxes, inputFluxes, typeNamesList, inputImageUnRedshifted

    def run_single_redshift(self):
        spectrumInfo = self._input_spectrum_single_redshift()
        self.emit(SIGNAL('load_spectrum_single_redshift(PyQt_PyObject)'), spectrumInfo)


    def run(self):
        if self.redshiftFlag == True:
            spectrumInfo = self._input_spectrum_single_redshift()
            self.emit(SIGNAL('load_spectrum_single_redshift(PyQt_PyObject)'), spectrumInfo)
        else:
            spectrumInfo = self._input_spectrum()
            self.emit(SIGNAL('load_spectrum(PyQt_PyObject)'), spectrumInfo)
        







def main():
    app = QtGui.QApplication(sys.argv)
    form = MainApp()
    form.show()
    app.exec_()

if __name__ == '__main__':
    main()



import numpy as np
from specutils.io import read_fits
from scipy.interpolate import interp1d, UnivariateSpline


class ProcessingTools(object):
    def __init__(self, nw):
        self.nw = nw

    def redshift_spectrum(self, wave, flux, z):
        wave_new = wave * (z + 1)

        return wave_new, flux

    def deredshift_spectrum(self, wave, flux, z):
        wave_new = wave / (z + 1)

        return wave_new, flux

    def min_max_index(self, flux):
        minindex, maxindex = (0, self.nw - 1)
        zeros = np.where(flux == 0)[0]
        j = 0
        for i in zeros:
            if (i != j):
                break
            j += 1
            minindex = j
        j = int(self.nw) - 1
        for i in zeros[::-1]:
            if (i != j):
                break
            j -= 1
            maxindex = j

        return minindex, maxindex


class ReadSpectrumFile(object):

    def __init__(self, filename, w0, w1, nw):
        self.filename = filename
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.processingTools = ProcessingTools(nw)

    def read_fits_file(self):
        #filename = unicode(self.filename.toUtf8(), encoding="UTF-8")
        spectrum = read_fits.read_fits_spectrum1d(self.filename)
        wave = np.array(spectrum.wavelength)
        flux = np.array(spectrum.flux)
        flux[np.isnan(flux)] = 0 #convert nan's to zeros

        return wave, flux

    def read_dat_file(self):
        wave = []
        flux = []
        try:
            with open(self.filename) as FileObj:
                for line in FileObj:
                    datapoint = line.rstrip('\n').strip().split()
                    if (len(datapoint) >= 2):
                        wave.append(float(datapoint[0].replace('D', 'E')))
                        flux.append(float(datapoint[1].replace('D', 'E')))
        except ValueError:
            print ("Invalid Superfit file: " + self.filename) #D-13 instead of E-13

        wave = np.array(wave)
        flux = np.array(flux)

        return wave, flux

    def file_extension(self):
        extension = self.filename.split('.')[-1]
        if extension == 'dat' or extension == self.filename:
            return self.read_dat_file()
        elif extension == 'fits':
            return self.read_fits_file()
        elif extension == 'lnw':
            return self.snid_template_spectra_all()
        else:
            print("Invalid Input File")
            return 0


    def two_col_input_spectrum(self, wave, flux, z):
        wave, flux = self.processingTools.deredshift_spectrum(wave, flux, z)

        if max(wave) >= self.w1:
            for i in range(0,len(wave)):
                if (wave[i] >= self.w1):
                    break
            wave = wave[0:i]
            flux = flux[0:i]
        if min(wave) < self.w0:
            for i in range(0,len(wave)):
                if (wave[i] >= self.w0):
                    break
            wave = wave[i:]
            flux = flux[i:]

        fluxNorm = (flux - min(flux))/(max(flux)-min(flux))

        return wave, fluxNorm

    def snid_template_spectra_all(self):
        """lnw file"""
        with open(self.filename) as FileObj:
            for i, line in enumerate(FileObj):
                # Read Header Info
                if i == 0:
                    header = (line.strip('\n')).split(' ')
                    header = [x for x in header if x != '']
                    numAges, nwx, w0x, w1x, mostknots, tname, dta, ttype, ittype, itstype = header
                    numAges, mostknots = map(int, (numAges, mostknots))
                    nk = np.zeros(numAges)
                    fmean = np.zeros(numAges)
                    xk = np.zeros((mostknots,numAges))
                    yk = np.zeros((mostknots,numAges))

                # Read Spline Info
                if i == 1:
                    splineInfo = (line.strip('\n')).split(' ')
                    splineInfo = [x for x in splineInfo if x != '']
                    for j in range(numAges):
                        nk[j], fmean[j] = (splineInfo[2*j+1], splineInfo[2*j+2])
                if i in range(2, mostknots+2):
                    splineInfo = (line.strip('\n')).split(' ')
                    splineInfo = [x for x in splineInfo if x != '']
                    for j in range(numAges):
                        xk[i-2,j], yk[i-2,j] = (splineInfo[2*j+1], splineInfo[2*j+2])

                if i == mostknots+2:
                    break

        splineInfo = (nk, fmean, xk, yk)

        # Read Normalized spectra
        arr=np.loadtxt(self.filename, skiprows=mostknots+2)
        ages = arr[0]
        ages = np.delete(ages, 0)
        arr = np.delete(arr, 0 ,0)

        wave = arr[:,0]
        fluxes = np.zeros(shape=(numAges,len(arr))) # initialise 2D array

        for i in range(0, len(arr[0])-1):
            fluxes[i] = arr[:,i+1]

        if ttype == 'Ia-99aa':
            ttype = 'Ia-91T'

        return wave, fluxes, numAges, ages, ttype, splineInfo



    def snid_template_spectra(self, wave, flux, z, splineInfo):
        #Undo Binning function -> then add galaxy -> then redshift

        waveRedshifted, fluxRedshifted = self.processingTools.redshift_spectrum(wave, flux, z)

        return waveRedshifted, fluxRedshifted

    

class PreProcessSpectrum(object):
    def __init__(self, w0, w1, nw):
        self.w0 = w0
        self.w1 = w1
        self.nw = nw
        self.dwlog = np.log(w1/w0) / nw
        self.processingTools = ProcessingTools(nw)

    def log_wavelength(self, wave, flux):
        fluxout = np.zeros(int(self.nw))
        j =0
        # Set up log wavelength array bins
        wlog = self.w0 * np.exp(np.arange(0,self.nw) * self.dwlog)

        A = self.nw/np.log(self.w1/self.w0)
        B = -self.nw*np.log(self.w0)/np.log(self.w1/self.w0)

        binnedwave = A*np.log(wave) + B

        # Rebin wavelengths
        for i in range(0,len(wave)):
            if (i == 0):
                s0 = 0.5*(3*wave[i] - wave[i+1])
                s1 = 0.5*(wave[i] + wave[i+1])
            elif (i == len(wave) - 1):
                s0 = 0.5*(wave[i-1] + wave[i])
                s1 = 0.5*(3*wave[i] - wave[i-1])
            else:
                s0 = 0.5 * (wave[i-1] + wave[i])
                s1 = 0.5 * (wave[i] + wave[i+1])

            s0log = np.log(s0/self.w0)/self.dwlog + 1
            s1log = np.log(s1/self.w0)/self.dwlog + 1
            dnu = s1-s0

            for j in range(int(s0log), int(s1log)):
                if (j < 1 or j >= self.nw):
                    continue
                alen = 1#min(s1log, j+1) - max(s0log, j)
                fluxval = flux[i] * alen/(s1log-s0log) * dnu
                fluxout[j] = fluxout[j] + fluxval



    ##            print (j, range(int(s0log), int(s1log)), int(s0log), s0log, int(s1log), s1log)
    ##            print fluxout[j]
    ##            print (j+1, s1log, j, s0log)
    ##            print (min(s1log, j+1), max(s0log, j), alen, s1log-s0log)
    ##            print ('--------------------------')


        minindex, maxindex = self.processingTools.min_max_index(fluxout)


        return wlog, fluxout, minindex, maxindex


    def spline_fit(self, wave, flux, numSplinePoints, minindex, maxindex):
        continuum = np.zeros(int(self.nw))
        spline = UnivariateSpline(wave[minindex:maxindex+1], flux[minindex:maxindex+1], k=3)
        splineWave = np.linspace(wave[minindex], wave[maxindex], num=numSplinePoints, endpoint=True)
        splinePoints = spline(splineWave)

        splineMore = UnivariateSpline(splineWave, splinePoints, k=3)
        splinePointsMore = splineMore(wave[minindex:maxindex])

        continuum[minindex:maxindex] = splinePointsMore

        return continuum


    def continuum_removal(self, wave, flux, numSplinePoints, minindex, maxindex):
        newflux = np.zeros(int(self.nw))

        splineFit = self.spline_fit(wave, flux, numSplinePoints, minindex, maxindex)
        newflux[minindex:maxindex] = flux[minindex:maxindex] - splineFit[minindex:maxindex]

        return newflux, splineFit


    def mean_zero(self, wave, flux, minindex, maxindex):
        """mean zero flux"""
        meanflux = np.mean(flux[minindex:maxindex])
        meanzeroflux = flux - meanflux

        for i in range(0,minindex):
            meanzeroflux[i] = 0
        for i in range(maxindex,len(flux)):
            meanzeroflux[i] = 0

        return meanzeroflux

    def apodize(self, wave, flux, minindex, maxindex):
        """apodize with 5% cosine bell"""
        percent = 0.05
        fluxout = flux + 0

        nsquash = int(self.nw*percent)
        for i in range(0, nsquash):
            arg = np.pi * i/(nsquash-1)
            factor = 0.5*(1-np.cos(arg))
            fluxout[minindex+i] = factor*fluxout[minindex+i]
            fluxout[maxindex-i] = factor*fluxout[maxindex-i]

        return fluxout



